#pragma region Copyright (c) 2014-2017 OpenRCT2 Developers
/*****************************************************************************
 * OpenRCT2, an open source clone of Roller Coaster Tycoon 2.
 *
 * OpenRCT2 is the work of many authors, a full list can be found in contributors.md
 * For more information, visit https://github.com/OpenRCT2/OpenRCT2
 *
 * OpenRCT2 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * A full copy of the GNU General Public License can be found in licence.txt
 *****************************************************************************/
#pragma endregion

#include "../paint/paint.h"
#include "../drawing/drawing.h"
#include "../world/sprite.h"
#include "../ride/ride_data.h"
#include "../ride/track.h"
#include "../interface/viewport.h"
#include "../game.h"
#include "../drawing/lightfx.h"
#include "vehicle_paint.h"

// 0x0098E52C:
const vehicle_boundbox VehicleBoundboxes[16][224] = {
    { // 0x0099123C:
    {  -5,  -2, -22, 10,  4, 14 }, {  -3,  -3, -22,  6,  6, 14 }, {  -3,  -3, -22,  6,  6, 14 }, {  -3,  -3, -22,  6,  6, 14 },
    {  -2,  -5, -22,  4, 10, 14 }, {  -3,  -3, -22,  6,  6, 14 }, {  -3,  -3, -22,  6,  6, 14 }, {  -3,  -3, -22,  6,  6, 14 },
    {  -5,  -2, -22, 10,  4, 14 }, {  -3,  -3, -22,  6,  6, 14 }, {  -3,  -3, -22,  6,  6, 14 }, {  -3,  -3, -22,  6,  6, 14 },
    {  -2,  -5, -22,  4, 10, 14 }, {  -3,  -3, -22,  6,  6, 14 }, {  -3,  -3, -22,  6,  6, 14 }, {  -3,  -3, -22,  6,  6, 14 },
    {  -5,  -2, -22, 10,  4, 14 }, {  -3,  -3, -22,  6,  6, 14 }, {  -3,  -3, -22,  6,  6, 14 }, {  -3,  -3, -22,  6,  6, 14 },
    {  -2,  -5, -22,  4, 10, 14 }, {  -3,  -3, -22,  6,  6, 14 }, {  -3,  -3, -22,  6,  6, 14 }, {  -3,  -3, -22,  6,  6, 14 },
    {  -5,  -2, -22, 10,  4, 14 }, {  -3,  -3, -22,  6,  6, 14 }, {  -3,  -3, -22,  6,  6, 14 }, {  -3,  -3, -22,  6,  6, 14 },
    {  -2,  -5, -22,  4, 10, 14 }, {  -3,  -3, -22,  6,  6, 14 }, {  -3,  -3, -22,  6,  6, 14 }, {  -3,  -3, -22,  6,  6, 14 },
    { -10,  -6, -22, 16, 12, 14 }, {  -8,  -8, -22, 16, 16, 14 }, {  -6, -11, -27,  3, 16, 19 }, {  -8,  -8, -22, 16, 16, 14 },
    { -11,  -6, -27, 16,  5, 19 }, {  -8,  -8, -22, 16, 16, 14 }, {  -6, -10, -22, 12, 16, 14 }, {  -8,  -8, -22, 16, 16, 14 },
    { -10,  -6, -22, 16, 12, 14 }, {  -6,  -6, -22, 12, 12, 14 }, {  -6,  -6, -22, 12, 12, 14 }, {  -6,  -6, -22, 12, 12, 14 },
    {  -3, -10, -33,  3, 16, 27 }, {  -6,  -6, -22, 12, 12, 14 }, {  -6,  -6, -22, 12, 12, 14 }, {  -6,  -6, -22, 12, 12, 14 },
    { -13,  -6, -33, 16,  5, 24 }, {  -6,  -6, -22, 12, 12, 14 }, {  -6,  -6, -22, 12, 12, 14 }, {  -6,  -6, -22, 12, 12, 14 },
    {  -6, -10, -22, 12, 16, 14 }, {  -6,  -6, -22, 12, 12, 14 }, {  -6,  -6, -22, 12, 12, 14 }, {  -6,  -6, -22, 12, 12, 14 },
    { -10,  -6, -33, 20, 12, 24 }, {  -2,  -3, -33,  3, 10, 29 }, { -13,  -6, -33, 20,  5, 24 }, {  -6, -10, -22, 12, 20, 14 },
    { -10,  -6, -43,  5, 12, 34 }, { -10,  -6, -43,  5, 12, 34 }, { -10,  -6, -43,  5, 12, 34 }, { -10,  -6, -43,  5, 12, 34 },
    {  -2,   4, -33,  3, 10, 29 }, {  -2,   4, -33,  3, 10, 29 }, {  -2,   4, -33,  3, 10, 29 }, {  -2,   4, -33,  3, 10, 29 },
    { -13,  -6, -33, 20,  5, 24 }, { -13,  -6, -33, 20,  5, 24 }, { -13,  -6, -33, 20,  5, 24 }, { -13,  -6, -33, 20,  5, 24 },
    {  -6, -15, -22, 12, 10, 14 }, {  -6, -15, -22, 12, 10, 14 }, {  -6, -15, -22, 12, 10, 14 }, {  -6, -15, -22, 12, 10, 14 },
    { -10,  -6, -33,  4, 12, 24 }, {  -2,   4, -10,  3, 10, 24 }, {  -3,  -6, -33, 10,  5, 24 }, {  -6, -15, -10, 12, 10, 14 },
    { -10,  -6, -22,  3, 12, 14 }, {  -6,   0, -10,  3, 10, 24 }, {  -3,  -6, -22, 10,  5, 14 }, {  -6, -15, -10, 12, 10, 14 },
    { -10,  -6, -38, 20, 12,  7 }, {  -6,   0,  -5, 12, 10,  7 }, {   0,  -6, -28, 10, 12,  7 }, {  -6, -15,  -5, 12, 10,  7 },
    { -10,  -6,   5, 20, 12,  7 }, {  -6,   0,   5, 12, 10,  7 }, {   0,  -6,   5, 10, 12,  7 }, {  -6, -10,   5, 12, 10,  7 },
    { -10,  -6,   5, 20, 12,  7 }, {  -6, -10,   5, 12, 20,  7 }, { -10,  -6,   5, 20, 12,  7 }, {  -6, -10,   5, 12, 20,  7 },
    { -10,  -6,   5, 20, 12,  7 }, {  -6, -10,   5, 12, 20,  7 }, { -10,  -6,   5, 20, 12,  7 }, {  -6, -10,   5, 12, 20,  7 },
    {  -8,  -8, -22, 16, 16, 14 }, {  -8,  -8, -22, 16, 16, 14 }, {  -8,  -8, -22, 16, 16, 14 }, {  -8,  -8, -22, 16, 16, 14 },
    {  -8,  -8, -22, 16, 16, 14 }, {  -8,  -8, -22, 16, 16, 14 }, {  -8,  -8, -22, 16, 16, 14 }, {  -8,  -8, -22, 16, 16, 14 },
    {  -8,  -6,  11, 16, 12,  4 }, {  -8,  -6,  11, 16, 12,  4 }, {  -8,  -8,  11, 16, 16,  4 }, {  -6,  -8,  11, 12, 16,  4 },
    {  -6,  -8,  11, 12, 16,  4 }, {  -6,  -8, -22, 12, 16, 14 }, {  -8,  -8, -22, 16, 16, 14 }, {  -8,  -6, -22, 16, 12, 14 },
    {  -8,  -6, -22, 16, 12, 14 }, {  -8,  -6, -22, 16, 12, 14 }, {  -8,  -8, -22, 16, 16, 14 }, {  -6,  -8, -22, 12, 16, 14 },
    {  -6,  -8, -22, 12, 16, 14 }, {  -6,  -8, -22, 12, 16, 14 }, {  -8,  -8, -22, 16, 16, 14 }, {  -8,  -6, -22, 16, 12, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6, -22, 20, 12, 14 }, {  -6, -10, -22, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6, -22, 20, 12, 14 }, {  -6, -10, -22, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6, -22, 20, 12, 14 }, {  -6, -10, -22, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6, -22, 20, 12, 14 }, {  -6, -10, -22, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 },
    { -10,  -6, -22, 20, 12, 14 }, {  -6, -10, -22, 12, 20, 14 }, { -10,  -6, -22, 20, 12, 14 }, {  -6, -10, -22, 12, 20, 14 },
    { -10,  -6, -22, 20, 12, 14 }, {  -6, -10, -22, 12, 20, 14 }, { -10,  -6,  11, 20, 12, 14 }, {  -6, -10, -22, 12, 20, 14 },
    { -10,  -6, -22, 20, 12, 14 }, {  -6, -10, -22, 12, 20, 14 }, { -10,  -6,  11, 20, 12, 14 }, {  -6, -10, -22, 12, 20, 14 },
    { -10,  -6, -22, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 },
    { -10,  -6, -22, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 },
    { -10,  -6, -22, 20, 12, 14 }, {  -6, -10, -22, 12, 20, 14 }, { -10,  -6, -22, 20, 12, 14 }, {  -6, -10, -22, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10, -22, 12, 20, 14 }, { -10,  -6, -22, 20, 12, 14 }, {  -6, -10, -22, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10, -22, 12, 20, 14 }, { -10,  -6, -22, 20, 12, 14 }, {  -6, -10, -22, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6, -22, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6, -22, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 },
    { -10,  -6, -22, 20, 12, 14 }, {  -6, -10, -22, 12, 20, 14 }, { -10,  -6, -22, 20, 12, 14 }, {  -6, -10, -22, 12, 20, 14 },
    { -10,  -6, -22, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6, -22, 20, 12, 14 }, {  -6, -10, -22, 12, 20, 14 },
    { -10,  -6, -22, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6, -22, 20, 12, 14 }, {  -6, -10, -22, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6,  11, 20, 12, 14 }, {  -6, -10, -22, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6,  11, 20, 12, 14 }, {  -6, -10, -22, 12, 20, 14 },
    { -10,  -6, -22, 20, 12, 14 }, {  -6, -10, -22, 12, 20, 14 }, { -10,  -6, -22, 20, 12, 14 }, {  -6, -10, -22, 12, 20, 14 },
    { -10,  -6, -22, 20, 12, 14 }, {  -6, -10, -22, 12, 20, 14 }, { -10,  -6, -22, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 },
    { -10,  -6, -22, 20, 12, 14 }, {  -6, -10, -22, 12, 20, 14 }, { -10,  -6, -22, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10, -22, 12, 20, 14 }, { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10, -22, 12, 20, 14 }, { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 },
    },{ // 0x0099027C:
    {  -8,  -6, -28, 16, 12, 14 }, {  -8,  -6, -28, 16, 12, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -6,  -8, -28, 12, 16, 14 },
    {  -6,  -8, -28, 12, 16, 14 }, {  -6,  -8, -28, 12, 16, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -8,  -6, -28, 16, 12, 14 },
    {  -8,  -6, -28, 16, 12, 14 }, {  -8,  -6, -28, 16, 12, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -6,  -8, -28, 12, 16, 14 },
    {  -6,  -8, -28, 12, 16, 14 }, {  -6,  -8, -28, 12, 16, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -8,  -6, -28, 16, 12, 14 },
    {  -6,  -6, -28, 12, 12, 14 }, {  -9,  -6, -28, 18, 12, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -6,  -9, -28, 12, 18, 14 },
    {  -6,  -6, -28, 12, 12, 14 }, {  -6,  -9, -28, 12, 18, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -9,  -6, -28, 18, 12, 14 },
    {  -6,  -6, -28, 12, 12, 14 }, {  -9,  -6, -28, 18, 12, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -6,  -9, -28, 12, 18, 14 },
    {  -6,  -6, -28, 12, 12, 14 }, {  -6,  -9, -28, 12, 18, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -9,  -6, -28, 18, 12, 14 },
    { -10,  -6, -28, 16, 12, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -6, -11, -33,  3, 16, 19 }, {  -8,  -8, -28, 16, 16, 14 },
    { -11,  -6, -33, 16,  5, 19 }, {  -8,  -8, -28, 16, 16, 14 }, {  -6, -10, -28, 12, 16, 14 }, {  -8,  -8, -28, 16, 16, 14 },
    { -10,  -6, -28, 16, 12, 14 }, {  -6,  -6, -28, 12, 12, 14 }, {  -6,  -6, -28, 12, 12, 14 }, {  -6,  -6, -28, 12, 12, 14 },
    {  -3, -10, -39,  3, 16, 27 }, {  -6,  -6, -28, 12, 12, 14 }, {  -6,  -6, -28, 12, 12, 14 }, {  -6,  -6, -28, 12, 12, 14 },
    { -10,  -6, -39, 16,  3, 27 }, {  -6,  -6, -28, 12, 12, 14 }, {  -6,  -6, -28, 12, 12, 14 }, {  -6,  -6, -28, 12, 12, 14 },
    {  -6, -10, -28, 12, 16, 14 }, {  -6,  -6, -28, 12, 12, 14 }, {  -6,  -6, -28, 12, 12, 14 }, {  -6,  -6, -28, 12, 12, 14 },
    { -10,  -6, -28, 20, 12, 14 }, {  -2,  -3, -39,  3, 10, 29 }, {  -3,  -2, -39, 10,  3, 29 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6, -49,  5, 12, 34 }, { -10,  -6, -49,  5, 12, 34 }, { -10,  -6, -49,  5, 12, 34 }, { -10,  -6, -49,  5, 12, 34 },
    {  -2,   4, -39,  3, 10, 29 }, {  -2,   4, -39,  3, 10, 29 }, {  -2,   4, -39,  3, 10, 29 }, {  -2,   4, -39,  3, 10, 29 },
    { -13,  -6, -39, 20,  5, 24 }, { -13,  -6, -39, 20,  5, 24 }, { -13,  -6, -39, 20,  5, 24 }, { -13,  -6, -39, 20,  5, 24 },
    {  -6, -15, -28, 12, 10, 14 }, {  -6, -15, -28, 12, 10, 14 }, {  -6, -15, -28, 12, 10, 14 }, {  -6, -15, -28, 12, 10, 14 },
    { -10,  -6, -39,  4, 12, 24 }, {  -2,   4, -10,  3, 10, 24 }, {  -3,  -6, -39, 10,  5, 24 }, {  -6, -15, -10, 12, 10, 14 },
    { -10,  -6, -28,  3, 12, 14 }, {  -6,   0, -10,  3, 10, 24 }, {  -3,  -6, -28, 10,  5, 14 }, {  -6, -15, -10, 12, 10, 14 },
    { -10,  -6, -44, 20, 12,  7 }, {  -6,   0,  -5, 12, 10,  7 }, {   0,  -6, -34, 10, 12,  7 }, {  -6, -15,  -5, 12, 10,  7 },
    { -10,  -6,   5, 20, 12,  7 }, {  -6,   0,   5, 12, 10,  7 }, {   0,  -6,   5, 10, 12,  7 }, {  -6, -10,   5, 12, 10,  7 },
    { -10,  -6,   5, 20, 12,  7 }, {  -6, -10,   5, 12, 20,  7 }, { -10,  -6,   5, 20, 12,  7 }, {  -6, -10,   5, 12, 20,  7 },
    { -10,  -6,   5, 20, 12,  7 }, {  -6, -10,   5, 12, 20,  7 }, { -10,  -6,   5, 20, 12,  7 }, {  -6, -10,   5, 12, 20,  7 },
    {  -8,  -8, -28, 16, 16, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -8,  -8, -28, 16, 16, 14 },
    {  -8,  -8, -28, 16, 16, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -8,  -8, -28, 16, 16, 14 },
    {  -8,  -6,  11, 16, 12,  4 }, {  -8,  -6,  11, 16, 12,  4 }, {  -8,  -8,  11, 16, 16,  4 }, {  -6,  -8,  11, 12, 16,  4 },
    {  -6,  -8,  11, 12, 16,  4 }, {  -6,  -8, -28, 12, 16, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -8,  -6, -28, 16, 12, 14 },
    {  -8,  -6, -28, 16, 12, 14 }, {  -8,  -6, -28, 16, 12, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -6,  -8, -28, 12, 16, 14 },
    {  -6,  -8, -28, 12, 16, 14 }, {  -6,  -8, -28, 12, 16, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -8,  -6, -28, 16, 12, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 },
    { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 }, { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 }, { -10,  -6,  11, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 }, { -10,  -6,  11, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6, -28, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 },
    { -10,  -6, -28, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 },
    { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 }, { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 }, { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 }, { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6, -28, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6, -28, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 },
    { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 }, { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6, -28, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6, -28, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6,  11, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6,  11, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 }, { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 }, { -10,  -6, -28, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 },
    { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 }, { -10,  -6, -28, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 }, { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 }, { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 },
    },{ // 0x009907BC:
    {  -8,  -6, -23, 16, 12, 14 }, {  -8,  -6, -23, 16, 12, 14 }, {  -8,  -8, -23, 16, 16, 14 }, {  -6,  -8, -23, 12, 16, 14 },
    {  -6,  -8, -23, 12, 16, 14 }, {  -6,  -8, -23, 12, 16, 14 }, {  -8,  -8, -23, 16, 16, 14 }, {  -8,  -6, -23, 16, 12, 14 },
    {  -8,  -6, -23, 16, 12, 14 }, {  -8,  -6, -23, 16, 12, 14 }, {  -8,  -8, -23, 16, 16, 14 }, {  -6,  -8, -23, 12, 16, 14 },
    {  -6,  -8, -23, 12, 16, 14 }, {  -6,  -8, -23, 12, 16, 14 }, {  -8,  -8, -23, 16, 16, 14 }, {  -8,  -6, -23, 16, 12, 14 },
    {  -6,  -6, -23, 12, 12, 14 }, {  -9,  -6, -23, 18, 12, 14 }, {  -8,  -8, -23, 16, 16, 14 }, {  -6,  -9, -23, 12, 18, 14 },
    {  -6,  -6, -23, 12, 12, 14 }, {  -6,  -9, -23, 12, 18, 14 }, {  -8,  -8, -23, 16, 16, 14 }, {  -9,  -6, -23, 18, 12, 14 },
    {  -6,  -6, -23, 12, 12, 14 }, {  -9,  -6, -23, 18, 12, 14 }, {  -8,  -8, -23, 16, 16, 14 }, {  -6,  -9, -23, 12, 18, 14 },
    {  -6,  -6, -23, 12, 12, 14 }, {  -6,  -9, -23, 12, 18, 14 }, {  -8,  -8, -23, 16, 16, 14 }, {  -9,  -6, -23, 18, 12, 14 },
    { -10,  -6, -23, 16, 12, 14 }, {  -8,  -8, -23, 16, 16, 14 }, {  -6, -11, -28,  3, 16, 19 }, {  -8,  -8, -23, 16, 16, 14 },
    { -11,  -6, -28, 16,  5, 19 }, {  -8,  -8, -23, 16, 16, 14 }, {  -6, -10, -23, 12, 16, 14 }, {  -8,  -8, -23, 16, 16, 14 },
    { -10,  -6, -23, 16, 12, 14 }, {  -6,  -6, -23, 12, 12, 14 }, {  -6,  -6, -23, 12, 12, 14 }, {  -6,  -6, -23, 12, 12, 14 },
    {  -3, -10, -34,  3, 16, 27 }, {  -6,  -6, -23, 12, 12, 14 }, {  -6,  -6, -23, 12, 12, 14 }, {  -6,  -6, -23, 12, 12, 14 },
    { -13,  -6, -34, 16,  5, 24 }, {  -6,  -6, -23, 12, 12, 14 }, {  -6,  -6, -23, 12, 12, 14 }, {  -6,  -6, -23, 12, 12, 14 },
    {  -6, -10, -23, 12, 16, 14 }, {  -6,  -6, -23, 12, 12, 14 }, {  -6,  -6, -23, 12, 12, 14 }, {  -6,  -6, -23, 12, 12, 14 },
    { -10,  -6, -34, 20, 12, 24 }, {  -2,  -3, -34,  3, 10, 29 }, { -13,  -6, -34, 20,  5, 24 }, {  -6, -10, -23, 12, 20, 14 },
    { -10,  -6, -44,  5, 12, 34 }, { -10,  -6, -44,  5, 12, 34 }, { -10,  -6, -44,  5, 12, 34 }, { -10,  -6, -44,  5, 12, 34 },
    {  -2,   4, -34,  3, 10, 29 }, {  -2,   4, -34,  3, 10, 29 }, {  -2,   4, -34,  3, 10, 29 }, {  -2,   4, -34,  3, 10, 29 },
    { -13,  -6, -34, 20,  5, 24 }, { -13,  -6, -34, 20,  5, 24 }, { -13,  -6, -34, 20,  5, 24 }, { -13,  -6, -34, 20,  5, 24 },
    {  -6, -15, -23, 12, 10, 14 }, {  -6, -15, -23, 12, 10, 14 }, {  -6, -15, -23, 12, 10, 14 }, {  -6, -15, -23, 12, 10, 14 },
    { -10,  -6, -34,  4, 12, 24 }, {  -2,   4, -10,  3, 10, 24 }, {  -3,  -6, -34, 10,  5, 24 }, {  -6, -15, -10, 12, 10, 14 },
    { -10,  -6, -23,  3, 12, 14 }, {  -6,   0, -10,  3, 10, 24 }, {  -3,  -6, -23, 10,  5, 14 }, {  -6, -15, -10, 12, 10, 14 },
    { -10,  -6, -39, 20, 12,  7 }, {  -6,   0,  -5, 12, 10,  7 }, {   0,  -6, -29, 10, 12,  7 }, {  -6, -15,  -5, 12, 10,  7 },
    { -10,  -6,   5, 20, 12,  7 }, {  -6,   0,   5, 12, 10,  7 }, {   0,  -6,   5, 10, 12,  7 }, {  -6, -10,   5, 12, 10,  7 },
    { -10,  -6,   5, 20, 12,  7 }, {  -6, -10,   5, 12, 20,  7 }, { -10,  -6,   5, 20, 12,  7 }, {  -6, -10,   5, 12, 20,  7 },
    { -10,  -6,   5, 20, 12,  7 }, {  -6, -10,   5, 12, 20,  7 }, { -10,  -6,   5, 20, 12,  7 }, {  -6, -10,   5, 12, 20,  7 },
    {  -8,  -8, -23, 16, 16, 14 }, {  -8,  -8, -23, 16, 16, 14 }, {  -8,  -8, -23, 16, 16, 14 }, {  -8,  -8, -23, 16, 16, 14 },
    {  -8,  -8, -23, 16, 16, 14 }, {  -8,  -8, -23, 16, 16, 14 }, {  -8,  -8, -23, 16, 16, 14 }, {  -8,  -8, -23, 16, 16, 14 },
    {  -8,  -6,  11, 16, 12,  4 }, {  -8,  -6,  11, 16, 12,  4 }, {  -8,  -8,  11, 16, 16,  4 }, {  -6,  -8,  11, 12, 16,  4 },
    {  -6,  -8,  11, 12, 16,  4 }, {  -6,  -8, -23, 12, 16, 14 }, {  -8,  -8, -23, 16, 16, 14 }, {  -8,  -6, -23, 16, 12, 14 },
    {  -8,  -6, -23, 16, 12, 14 }, {  -8,  -6, -23, 16, 12, 14 }, {  -8,  -8, -23, 16, 16, 14 }, {  -6,  -8, -23, 12, 16, 14 },
    {  -6,  -8, -23, 12, 16, 14 }, {  -6,  -8, -23, 12, 16, 14 }, {  -8,  -8, -23, 16, 16, 14 }, {  -8,  -6, -23, 16, 12, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 },
    { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 }, { -10,  -6,  11, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 }, { -10,  -6,  11, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    { -10,  -6, -23, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 },
    { -10,  -6, -23, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 },
    { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 },
    { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    { -10,  -6, -23, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    { -10,  -6, -23, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6,  11, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6,  11, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 },
    { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 }, { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 }, { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 },
    },{ // 0x009921FC:
    { -19,  -6, -28, 38, 12, 14 }, { -11,  -6, -28, 22, 12, 14 }, { -11, -11, -28, 22, 22, 14 }, {  -6, -11, -28, 12, 22, 14 },
    {  -6, -19, -28, 12, 38, 14 }, {  -6, -11, -28, 12, 22, 14 }, { -11, -11, -28, 22, 22, 14 }, { -11,  -6, -28, 22, 12, 14 },
    { -19,  -6, -28, 38, 12, 14 }, { -11,  -6, -28, 22, 12, 14 }, { -11, -11, -28, 22, 22, 14 }, {  -6, -11, -28, 12, 22, 14 },
    {  -6, -19, -28, 12, 38, 14 }, {  -6, -11, -28, 12, 22, 14 }, { -11, -11, -28, 22, 22, 14 }, { -11,  -6, -28, 22, 12, 14 },
    { -15,  -6, -28, 18, 12, 14 }, {  -9,  -6, -28, 18, 12, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -6,  -9, -28, 12, 18, 14 },
    {  -6,  -6, -28, 12, 28, 14 }, {  -6,  -9, -28, 12, 18, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -9,  -6, -28, 18, 12, 14 },
    {  -6,  -6, -28, 28, 12, 14 }, {  -9,  -6, -28, 18, 12, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -6,  -9, -28, 12, 18, 14 },
    {  -6, -15, -28, 12, 18, 14 }, {  -6,  -9, -28, 12, 18, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -9,  -6, -28, 18, 12, 14 },
    { -10,  -6, -28, 16, 12, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -6, -11, -33,  3, 16, 19 }, {  -8,  -8, -28, 16, 16, 14 },
    { -11,  -6, -33, 16,  5, 19 }, {  -8,  -8, -28, 16, 16, 14 }, {  -6, -10, -28, 12, 16, 14 }, {  -8,  -8, -28, 16, 16, 14 },
    { -10,  -6, -28, 16, 12, 14 }, {  -6,  -6, -28, 12, 12, 14 }, {  -6,  -6, -28, 12, 12, 14 }, {  -6,  -6, -28, 12, 12, 14 },
    {  -3, -10, -39,  3, 16, 27 }, {  -6,  -6, -28, 12, 12, 14 }, {  -6,  -6, -28, 12, 12, 14 }, {  -6,  -6, -28, 12, 12, 14 },
    { -13,  -6, -39, 16,  5, 24 }, {  -6,  -6, -28, 12, 12, 14 }, {  -6,  -6, -28, 12, 12, 14 }, {  -6,  -6, -28, 12, 12, 14 },
    {  -6, -10, -28, 12, 16, 14 }, {  -6,  -6, -28, 12, 12, 14 }, {  -6,  -6, -28, 12, 12, 14 }, {  -6,  -6, -28, 12, 12, 14 },
    { -10,  -6, -39, 20, 12, 24 }, {  -2,  -3, -39,  3, 10, 29 }, { -13,  -6, -39, 20,  5, 24 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6, -49,  5, 12, 34 }, { -10,  -6, -49,  5, 12, 34 }, { -10,  -6, -49,  5, 12, 34 }, { -10,  -6, -49,  5, 12, 34 },
    {  -2,   4, -39,  3, 10, 29 }, {  -2,   4, -39,  3, 10, 29 }, {  -2,   4, -39,  3, 10, 29 }, {  -2,   4, -39,  3, 10, 29 },
    { -13,  -6, -39, 20,  5, 24 }, { -13,  -6, -39, 20,  5, 24 }, { -13,  -6, -39, 20,  5, 24 }, { -13,  -6, -39, 20,  5, 24 },
    {  -6, -15, -28, 12, 10, 14 }, {  -6, -15, -28, 12, 10, 14 }, {  -6, -15, -28, 12, 10, 14 }, {  -6, -15, -28, 12, 10, 14 },
    { -10,  -6, -39,  4, 12, 24 }, {  -2,   4, -10,  3, 10, 24 }, {  -3,  -6, -39, 10,  5, 24 }, {  -6, -15, -10, 12, 10, 14 },
    { -10,  -6, -28,  3, 12, 14 }, {  -6,   0, -10,  3, 10, 24 }, {  -3,  -6, -28, 10,  5, 14 }, {  -6, -15, -10, 12, 10, 14 },
    { -10,  -6, -44, 20, 12,  7 }, {  -6,   0,  -5, 12, 10,  7 }, {   0,  -6, -34, 10, 12,  7 }, {  -6, -15,  -5, 12, 10,  7 },
    { -10,  -6,   5, 20, 12,  7 }, {  -6,   0,   5, 12, 10,  7 }, {   0,  -6,   5, 10, 12,  7 }, {  -6, -10,   5, 12, 10,  7 },
    { -10,  -6,   5, 20, 12,  7 }, {  -6, -10,   5, 12, 20,  7 }, { -10,  -6,   5, 20, 12,  7 }, {  -6, -10,   5, 12, 20,  7 },
    { -10,  -6,   5, 20, 12,  7 }, {  -6, -10,   5, 12, 20,  7 }, { -10,  -6,   5, 20, 12,  7 }, {  -6, -10,   5, 12, 20,  7 },
    {  -8,  -8, -28, 16, 16, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -8,  -8, -28, 16, 16, 14 },
    {  -8,  -8, -28, 16, 16, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -8,  -8, -28, 16, 16, 14 },
    {  -8,  -6,  11, 16, 12,  4 }, {  -8,  -6,  11, 16, 12,  4 }, {  -8,  -8,  11, 16, 16,  4 }, {  -6,  -8,  11, 12, 16,  4 },
    {  -6,  -8,  11, 12, 16,  4 }, {  -6,  -8, -28, 12, 16, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -8,  -6, -28, 16, 12, 14 },
    {  -8,  -6, -28, 16, 12, 14 }, {  -8,  -6, -28, 16, 12, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -6,  -8, -28, 12, 16, 14 },
    {  -6,  -8, -28, 12, 16, 14 }, {  -6,  -8, -28, 12, 16, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -8,  -6, -28, 16, 12, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 },
    { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 }, { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 }, { -10,  -6,  11, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 }, { -10,  -6,  11, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6, -28, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 },
    { -10,  -6, -28, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 },
    { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 }, { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 }, { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 }, { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6, -28, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6, -28, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 },
    { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 }, { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6, -28, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6, -28, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6,  11, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6,  11, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 }, { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 }, { -10,  -6, -28, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 },
    { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 }, { -10,  -6, -28, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 }, { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 }, { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 },
    },{ // 0x00991CBC:
    { -24,  -6, -28, 32, 12, 14 }, { -11,  -6, -28, 22, 12, 14 }, { -11, -11, -28, 22, 22, 14 }, {  -6, -11, -28, 12, 22, 14 },
    {  -6, -11, -28, 12, 32, 14 }, {  -6, -11, -28, 12, 22, 14 }, { -11, -11, -28, 22, 22, 14 }, { -11,  -6, -28, 22, 12, 14 },
    { -11,  -6, -28, 32, 12, 14 }, { -11,  -6, -28, 22, 12, 14 }, { -11, -11, -28, 22, 22, 14 }, {  -6, -11, -28, 12, 22, 14 },
    {  -6, -24, -28, 12, 32, 14 }, {  -6, -11, -28, 12, 22, 14 }, { -11, -11, -28, 22, 22, 14 }, { -11,  -6, -28, 22, 12, 14 },
    { -15,  -6, -28, 18, 12, 14 }, {  -9,  -6, -28, 18, 12, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -6,  -9, -28, 12, 18, 14 },
    {  -6,  -6, -28, 12, 28, 14 }, {  -6,  -9, -28, 12, 18, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -9,  -6, -28, 18, 12, 14 },
    {  -6,  -6, -28, 28, 12, 14 }, {  -9,  -6, -28, 18, 12, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -6,  -9, -28, 12, 18, 14 },
    {  -6, -15, -28, 12, 18, 14 }, {  -6,  -9, -28, 12, 18, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -9,  -6, -28, 18, 12, 14 },
    { -10,  -6, -28, 16, 12, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -6, -11, -33,  3, 16, 19 }, {  -8,  -8, -28, 16, 16, 14 },
    { -11,  -6, -33, 16,  5, 19 }, {  -8,  -8, -28, 16, 16, 14 }, {  -6, -10, -28, 12, 16, 14 }, {  -8,  -8, -28, 16, 16, 14 },
    { -10,  -6, -28, 16, 12, 14 }, {  -6,  -6, -28, 12, 12, 14 }, {  -6,  -6, -28, 12, 12, 14 }, {  -6,  -6, -28, 12, 12, 14 },
    {  -3, -10, -39,  3, 16, 27 }, {  -6,  -6, -28, 12, 12, 14 }, {  -6,  -6, -28, 12, 12, 14 }, {  -6,  -6, -28, 12, 12, 14 },
    { -13,  -6, -39, 16,  5, 24 }, {  -6,  -6, -28, 12, 12, 14 }, {  -6,  -6, -28, 12, 12, 14 }, {  -6,  -6, -28, 12, 12, 14 },
    {  -6, -10, -28, 12, 16, 14 }, {  -6,  -6, -28, 12, 12, 14 }, {  -6,  -6, -28, 12, 12, 14 }, {  -6,  -6, -28, 12, 12, 14 },
    { -10,  -6, -39, 20, 12, 24 }, {  -2,  -3, -39,  3, 10, 29 }, { -13,  -6, -39, 20,  5, 24 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6, -49,  5, 12, 34 }, { -10,  -6, -49,  5, 12, 34 }, { -10,  -6, -49,  5, 12, 34 }, { -10,  -6, -49,  5, 12, 34 },
    {  -2,   4, -39,  3, 10, 29 }, {  -2,   4, -39,  3, 10, 29 }, {  -2,   4, -39,  3, 10, 29 }, {  -2,   4, -39,  3, 10, 29 },
    { -13,  -6, -39, 20,  5, 24 }, { -13,  -6, -39, 20,  5, 24 }, { -13,  -6, -39, 20,  5, 24 }, { -13,  -6, -39, 20,  5, 24 },
    {  -6, -15, -28, 12, 10, 14 }, {  -6, -15, -28, 12, 10, 14 }, {  -6, -15, -28, 12, 10, 14 }, {  -6, -15, -28, 12, 10, 14 },
    { -10,  -6, -39,  4, 12, 24 }, {  -2,   4, -10,  3, 10, 24 }, {  -3,  -6, -39, 10,  5, 24 }, {  -6, -15, -10, 12, 10, 14 },
    { -10,  -6, -28,  3, 12, 14 }, {  -6,   0, -10,  3, 10, 24 }, {  -3,  -6, -28, 10,  5, 14 }, {  -6, -15, -10, 12, 10, 14 },
    { -10,  -6, -44, 20, 12,  7 }, {  -6,   0,  -5, 12, 10,  7 }, {   0,  -6, -34, 10, 12,  7 }, {  -6, -15,  -5, 12, 10,  7 },
    { -10,  -6,   5, 20, 12,  7 }, {  -6,   0,   5, 12, 10,  7 }, {   0,  -6,   5, 10, 12,  7 }, {  -6, -10,   5, 12, 10,  7 },
    { -10,  -6,   5, 20, 12,  7 }, {  -6, -10,   5, 12, 20,  7 }, { -10,  -6,   5, 20, 12,  7 }, {  -6, -10,   5, 12, 20,  7 },
    { -10,  -6,   5, 20, 12,  7 }, {  -6, -10,   5, 12, 20,  7 }, { -10,  -6,   5, 20, 12,  7 }, {  -6, -10,   5, 12, 20,  7 },
    {  -8,  -8, -28, 16, 16, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -8,  -8, -28, 16, 16, 14 },
    {  -8,  -8, -28, 16, 16, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -8,  -8, -28, 16, 16, 14 },
    {  -8,  -6,  11, 16, 12,  4 }, {  -8,  -6,  11, 16, 12,  4 }, {  -8,  -8,  11, 16, 16,  4 }, {  -6,  -8,  11, 12, 16,  4 },
    {  -6,  -8,  11, 12, 16,  4 }, {  -6,  -8, -28, 12, 16, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -8,  -6, -28, 16, 12, 14 },
    {  -8,  -6, -28, 16, 12, 14 }, {  -8,  -6, -28, 16, 12, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -6,  -8, -28, 12, 16, 14 },
    {  -6,  -8, -28, 12, 16, 14 }, {  -6,  -8, -28, 12, 16, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -8,  -6, -28, 16, 12, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 },
    { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 }, { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 }, { -10,  -6,  11, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 }, { -10,  -6,  11, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6, -28, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 },
    { -10,  -6, -28, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 },
    { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 }, { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 }, { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 }, { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6, -28, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6, -28, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 },
    { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 }, { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6, -28, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6, -28, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6,  11, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6,  11, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 }, { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 }, { -10,  -6, -28, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 },
    { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 }, { -10,  -6, -28, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 }, { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 }, { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 },
    },{ // 0x0098F52C
    { -14,  -6,   1, 28, 12, 14 }, { -12,  -6,   1, 24, 12, 14 }, { -10, -10,   1, 20, 20, 14 }, {  -6, -12,   1, 12, 24, 14 },
    {  -6, -14,   1, 12, 28, 14 }, {  -6, -12,   1, 12, 24, 14 }, { -10, -10,   1, 20, 20, 14 }, { -12,  -6,   1, 24, 12, 14 },
    { -14,  -6,   1, 28, 12, 14 }, { -12,  -6,   1, 24, 12, 14 }, { -10, -10,   1, 20, 20, 14 }, {  -6, -12,   1, 12, 24, 14 },
    {  -6, -14,   1, 12, 28, 14 }, {  -6, -12,   1, 12, 24, 14 }, { -10, -10,   1, 20, 20, 14 }, { -12,  -6,   1, 24, 12, 14 },
    { -13,  -6,   1, 26, 12, 14 }, { -11,  -6,   1, 22, 12, 14 }, {  -8,  -8,   1, 16, 16, 14 }, {  -6, -11,   1, 12, 22, 14 },
    {  -6, -13,   3, 12, 26, 14 }, {  -6, -11,   1, 12, 22, 14 }, {  -8,  -8,   1, 16, 16, 14 }, { -11,  -6,   1, 22, 12, 14 },
    { -13,  -6,   3, 26, 12, 14 }, { -11,  -6,   1, 22, 12, 14 }, {  -8,  -8,   1, 16, 16, 14 }, {  -6, -11,   1, 12, 22, 14 },
    {  -6, -13,   1, 12, 26, 14 }, {  -6, -11,   1, 12, 22, 14 }, {  -8,  -8,   1, 16, 16, 14 }, { -11,  -6,   1, 22, 12, 14 },
    { -14,  -6,   1, 28, 12, 14 }, {  -8,  -8,   1, 16, 16, 14 }, {  -6, -14,   1, 12, 28, 14 }, {  -8,  -8,   1, 16, 16, 14 },
    { -18,  -6,   1, 28, 12, 14 }, {  -8,  -8,   1, 16, 16, 14 }, {  -6, -14,   1, 12, 28, 14 }, {   0,   0,   1,  8,  8, 14 },
    {  -3,  -6,   5,  6, 12,  7 }, {  -8,  -6,   1, 16, 12,  7 }, {  -8,  -8,   1, 16, 16,  7 }, {  -6,  -8,   1, 12, 16,  7 },
    {  -6,  -8,   5, 12,  6,  7 }, {  -6,  -8,   1, 12, 16,  7 }, {  -8,  -8,   1, 16, 16,  7 }, {  -8,  -6,   1, 16, 12,  7 },
    {  -8,  -6,   5,  6, 12,  7 }, {  -8,  -6,   1, 16, 12,  7 }, {  -8,  -8,   1, 16, 16,  7 }, {  -6,  -8,   1, 12, 16,  7 },
    {  -6,  -3,   5, 12,  6,  7 }, {  -6,  -8,   1, 12, 16,  7 }, {  -8,  -8,   1, 16, 16,  7 }, {  -8,  -6,   1, 16, 12,  7 },
    {   0,  -6,   5,  3, 12,  5 }, {  -6, -13,   5,  3, 12,  5 }, { -13,  -6,   5, 12,  5,  5 }, {  -6,   0,   5, 12,  3,  5 },
    {   0,  -6,  -8, 10, 12, 15 }, {   0,  -6,  -8, 10, 12, 15 }, {   0,  -6,  -8, 10, 12, 15 }, {  -6, -12,  -8, 12, 10, 15 },
    {  -6, -12,  -8, 12, 10, 15 }, {  -6, -12,  -8, 12, 10, 15 }, {  -6, -12,  -8, 12, 10, 15 }, {  -6, -12,  -8, 12, 10, 15 },
    { -12,  -6,  -8, 10, 12, 15 }, { -12,  -6,  -8, 10, 12, 15 }, { -12,  -6,  -8, 10, 12, 15 }, {  -6,   0,  -8, 12, 10, 15 },
    {  -6,   0,  -8, 12, 10, 15 }, {  -6,   0,  -8, 12, 10, 15 }, {  -6,   0,  -8, 12, 10, 15 }, {  -6,   0,  -8, 12, 10, 15 },
    {  -3,  -6,  -4,  6, 12,  5 }, {  -6, -13,  -4,  3, 20,  5 }, { -13,  -6,  -4,  6,  5,  5 }, {  -6,  -3,  -4, 12, 20,  5 },
    {  -3,  -6,   1,  6, 12,  6 }, {  -6, -13, -10,  3,  6,  6 }, { -13,  -6, -10,  6,  5,  6 }, {  -6,  -3,   1, 12,  6,  6 },
    { -14,  -6, -15, 28, 12,  7 }, {  -6, -14, -15, 12, 28,  7 }, { -14,  -6, -15, 28, 12,  7 }, {  -6, -14, -15, 12, 28,  7 },
    { -14,  -6, -15, 28, 12,  7 }, {  -6, -14, -15, 12, 28,  7 }, { -14,  -6, -15, 28, 12,  7 }, {  -6, -14, -15, 12, 28,  7 },
    { -14,  -6, -15, 28, 12,  7 }, {  -6, -14, -15, 12, 28,  7 }, { -14,  -6, -15, 28, 12,  7 }, {  -6, -14, -15, 12, 28,  7 },
    { -14,  -6, -15, 28, 12,  7 }, {  -6, -14, -15, 12, 28,  7 }, { -14,  -6, -15, 28, 12,  7 }, {  -6, -14, -15, 12, 28,  7 },
    { -10, -10,   1, 20, 20, 14 }, { -10, -10,   1, 20, 20, 14 }, { -10, -10,   1, 20, 20, 14 }, { -10, -10,   1, 20, 20, 14 },
    {  -8,  -8,   1, 16, 16, 14 }, { -32, -32,  24,  4,  4, 14 }, {  -8,  -8,   1, 16, 16, 14 }, {   8,   8,   1,  8,  8, 14 },
    { -14,  -6,   1, 28, 12, 14 }, { -12,  -6,   1, 24, 12, 14 }, { -10, -10,   1, 20, 20, 14 }, {  -6, -12,   1, 12, 24, 14 },
    {  -6, -14,   1, 12, 28, 14 }, {  -6, -12,   1, 12, 24, 14 }, { -10, -10,   1, 20, 20, 14 }, { -12,  -6,   1, 24, 12, 14 },
    { -14,  -6,   1, 28, 12, 14 }, { -12,  -6,   1, 24, 12, 14 }, { -10, -10,   1, 20, 20, 14 }, {  -6, -12,   1, 12, 24, 14 },
    {  -6, -14,   1, 12, 28, 14 }, {  -6, -12,   1, 12, 24, 14 }, { -10, -10,   1, 20, 20, 14 }, { -12,  -6,   1, 24, 12, 14 },
    {  -6,  -6, -12, 12, 12,  2 }, {  -6,  -6, -12, 12, 12,  2 }, {  -6,  -6,  10, 12, 12, 14 }, {  -6,  -6,  10, 12, 12, 14 },
    {  -6,  -6, -12, 12, 12,  2 }, {  -6,  -6, -12, 12, 12,  2 }, {  -6,  -6,  10, 12, 12, 14 }, {  -6,  -6,  10, 12, 12, 14 },
    {  -6,  -6, -12, 12, 12,  2 }, {  -6,  -6, -12, 12, 12,  2 }, {  -6,  -6,  10, 12, 12, 14 }, {  -6,  -6,  10, 12, 12, 14 },
    {  -6,  -6, -12, 12, 12,  2 }, {  -6,  -6, -12, 12, 12,  2 }, {  -6,  -6,  10, 12, 12, 14 }, {  -6,  -6,  10, 12, 12, 14 },
    {  -6,  -6, -12, 12, 12,  2 }, {  -6,  -6, -12, 12, 12,  2 }, {  -6,  -6, -12, 12, 12,  2 }, {  -6,  -6, -12, 12, 12,  2 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6, -23, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6, -23, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    { -10,  -6, -23, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    { -10,  -6, -23, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    },{ // 0x0098EFEC:
    { -10,  -6,   1, 20, 12, 14 }, {  -9,  -6,   1, 18, 12, 14 }, {  -8,  -8,   1, 16, 16, 14 }, {  -6,  -9,   1, 12, 18, 14 },
    {  -6, -10,   1, 12, 20, 14 }, {  -6,  -9,   1, 12, 18, 14 }, {  -8,  -8,   1, 16, 16, 14 }, {  -9,  -6,   1, 18, 12, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -9,  -6,   1, 18, 12, 14 }, {  -8,  -8,   1, 16, 16, 14 }, {  -6,  -9,   1, 12, 18, 14 },
    {  -6, -10,   1, 12, 20, 14 }, {  -6,  -9,   1, 12, 18, 14 }, {  -8,  -8,   1, 16, 16, 14 }, {  -9,  -6,   1, 18, 12, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -8,  -6,   1, 16, 12, 14 }, {  -8,  -8,   1, 16, 16, 14 }, {  -6,  -8,   1, 12, 16, 14 },
    {  -6, -10,   2, 12, 20, 14 }, {  -6,  -8,   1, 12, 16, 14 }, {  -8,  -8,   1, 16, 16, 14 }, {  -8,  -6,   1, 16, 12, 14 },
    { -10,  -6,   2, 20, 12, 14 }, {  -8,  -6,   1, 16, 12, 14 }, {  -8,  -8,   1, 16, 16, 14 }, {  -6,  -8,   1, 12, 16, 14 },
    {  -6, -10,   1, 12, 20, 14 }, {  -6,  -8,   1, 12, 16, 14 }, {  -8,  -8,   1, 16, 16, 14 }, {  -8,  -6,   1, 16, 12, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -8,  -8,   1, 16, 16, 14 }, {  -6, -10,   1, 12, 20, 14 }, {  -8,  -8,   1, 16, 16, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -8,  -8,   1, 16, 16, 14 }, {  -6, -10,   1, 12, 20, 14 }, {   0,   0,   1,  8,  8, 14 },
    {  -3,  -6,   5,  6, 12,  7 }, {  -8,  -6,   1, 16, 12,  7 }, {  -8,  -8,   1, 16, 16,  7 }, {  -6,  -8,   1, 12, 16,  7 },
    {  -6,  -8,   5, 12,  6,  7 }, {  -6,  -8,   1, 12, 16,  7 }, {  -8,  -8,   1, 16, 16,  7 }, {  -8,  -6,   1, 16, 12,  7 },
    {  -8,  -6,   5,  6, 12,  7 }, {  -8,  -6,   1, 16, 12,  7 }, {  -8,  -8,   1, 16, 16,  7 }, {  -6,  -8,   1, 12, 16,  7 },
    {  -6,  -3,   5, 12,  6,  7 }, {  -6,  -8,   1, 12, 16,  7 }, {  -8,  -8,   1, 16, 16,  7 }, {  -8,  -6,   1, 16, 12,  7 },
    {   0,  -6,   5,  3, 12,  5 }, {  -6, -13,   5,  3, 12,  5 }, { -13,  -6,   5, 12,  5,  5 }, {  -6,   0,   5, 12,  3,  5 },
    {   0,  -6,  -8, 10, 12, 15 }, {   0,  -6,  -8, 10, 12, 15 }, {   0,  -6,  -8, 10, 12, 15 }, {  -6, -12,  -8, 12, 10, 15 },
    {  -6, -12,  -8, 12, 10, 15 }, {  -6, -12,  -8, 12, 10, 15 }, {  -6, -12,  -8, 12, 10, 15 }, {  -6, -12,  -8, 12, 10, 15 },
    { -12,  -6,  -8, 10, 12, 15 }, { -12,  -6,  -8, 10, 12, 15 }, { -12,  -6,  -8, 10, 12, 15 }, {  -6,   0,  -8, 12, 10, 15 },
    {  -6,   0,  -8, 12, 10, 15 }, {  -6,   0,  -8, 12, 10, 15 }, {  -6,   0,  -8, 12, 10, 15 }, {  -6,   0,  -8, 12, 10, 15 },
    {  -3,  -6,  -4,  6, 12,  5 }, {  -6, -13,  -4,  3, 20,  5 }, { -13,  -6,  -4,  6,  5,  5 }, {  -6,  -3,  -4, 12, 20,  5 },
    {  -3,  -6,   1,  6, 12,  6 }, {  -6, -13, -10,  3,  6,  6 }, { -13,  -6, -10,  6,  5,  6 }, {  -6,  -3,   1, 12,  6,  6 },
    { -10,  -6, -15, 20, 12,  7 }, {  -6, -10, -15, 12, 20,  7 }, { -10,  -6, -15, 20, 12,  7 }, {  -6, -10, -15, 12, 20,  7 },
    { -10,  -6, -15, 20, 12,  7 }, {  -6, -10, -15, 12, 20,  7 }, { -10,  -6, -15, 20, 12,  7 }, {  -6, -10, -15, 12, 20,  7 },
    { -10,  -6, -15, 20, 12,  7 }, {  -6, -10, -15, 12, 20,  7 }, { -10,  -6, -15, 20, 12,  7 }, {  -6, -10, -15, 12, 20,  7 },
    { -10,  -6, -15, 20, 12,  7 }, {  -6, -10, -15, 12, 20,  7 }, { -10,  -6, -15, 20, 12,  7 }, {  -6, -10, -15, 12, 20,  7 },
    {  -8,  -8,   1, 16, 16, 14 }, {  -8,  -8,   1, 16, 16, 14 }, {  -8,  -8,   1, 16, 16, 14 }, {  -8,  -8,   1, 16, 16, 14 },
    {  -8,  -8,   1, 16, 16, 14 }, { -32, -32,  24,  4,  4, 14 }, {  -8,  -8,   1, 16, 16, 14 }, {   8,   8,   1,  8,  8, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -9,  -6,   1, 18, 12, 14 }, {  -8,  -8,   1, 16, 16, 14 }, {  -6,  -9,   1, 12, 18, 14 },
    {  -6, -10,   1, 12, 20, 14 }, {  -6,  -9,   1, 12, 18, 14 }, {  -8,  -8,   1, 16, 16, 14 }, {  -9,  -6,   1, 18, 12, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -9,  -6,   1, 18, 12, 14 }, {  -8,  -8,   1, 16, 16, 14 }, {  -6,  -9,   1, 12, 18, 14 },
    {  -6, -10,   1, 12, 20, 14 }, {  -6,  -9,   1, 12, 18, 14 }, {  -8,  -8,   1, 16, 16, 14 }, {  -9,  -6,   1, 18, 12, 14 },
    {  -6,  -6, -12, 12, 12,  2 }, {  -6,  -6, -12, 12, 12,  2 }, {  -6,  -6,  10, 12, 12, 14 }, {  -6,  -6,  10, 12, 12, 14 },
    {  -6,  -6, -12, 12, 12,  2 }, {  -6,  -6, -12, 12, 12,  2 }, {  -6,  -6,  10, 12, 12, 14 }, {  -6,  -6,  10, 12, 12, 14 },
    {  -6,  -6, -12, 12, 12,  2 }, {  -6,  -6, -12, 12, 12,  2 }, {  -6,  -6,  10, 12, 12, 14 }, {  -6,  -6,  10, 12, 12, 14 },
    {  -6,  -6, -12, 12, 12,  2 }, {  -6,  -6, -12, 12, 12,  2 }, {  -6,  -6,  10, 12, 12, 14 }, {  -6,  -6,  10, 12, 12, 14 },
    {  -6,  -6, -12, 12, 12,  2 }, {  -6,  -6, -12, 12, 12,  2 }, {  -6,  -6, -12, 12, 12,  2 }, {  -6,  -6, -12, 12, 12,  2 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6, -23, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6, -23, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    { -10,  -6, -23, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    { -10,  -6, -23, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    },{ // 0x0098EAAC:
    {  -8,  -6,   1, 16, 12, 14 }, {  -7,  -6,   1, 14, 12, 14 }, {  -6,  -6,   1, 12, 12, 14 }, {  -6,  -7,   1, 12, 14, 14 },
    {  -6,  -8,   1, 12, 16, 14 }, {  -6,  -7,   1, 12, 14, 14 }, {  -6,  -6,   1, 12, 12, 14 }, {  -7,  -6,   1, 14, 12, 14 },
    {  -8,  -6,   1, 16, 12, 14 }, {  -7,  -6,   1, 14, 12, 14 }, {  -6,  -6,   1, 12, 12, 14 }, {  -6,  -7,   1, 12, 14, 14 },
    {  -6,  -8,   1, 12, 16, 14 }, {  -6,  -7,   1, 12, 14, 14 }, {  -6,  -6,   1, 12, 12, 14 }, {  -7,  -6,   1, 14, 12, 14 },
    {  -8,  -6,   1, 16, 12, 14 }, {  -7,  -6,   1, 14, 12, 14 }, {  -8,  -8,   1, 16, 16, 14 }, {  -6,  -7,   1, 12, 14, 14 },
    {  -6,  -8,   2, 12, 16, 14 }, {  -6,  -7,   1, 12, 14, 14 }, {  -8,  -8,   1, 16, 16, 14 }, {  -7,  -6,   1, 14, 12, 14 },
    {  -8,  -6,   2, 16, 12, 14 }, {  -7,  -6,   1, 14, 12, 14 }, {  -8,  -8,   1, 16, 16, 14 }, {  -6,  -7,   1, 12, 14, 14 },
    {  -6,  -8,   1, 12, 16, 14 }, {  -6,  -7,   1, 12, 14, 14 }, {  -8,  -8,   1, 16, 16, 14 }, {  -7,  -6,   1, 14, 12, 14 },
    {  -8,  -6,   1, 16, 12, 14 }, {  -8,  -8,   1, 16, 16, 14 }, {  -6,  -8,   1, 12, 16, 14 }, {  -8,  -8,   1, 16, 16, 14 },
    {  -8,  -6,   1, 16, 12, 14 }, {  -8,  -8,   1, 16, 16, 14 }, {  -6,  -8,   1, 12, 16, 14 }, {   0,   0,   1,  8,  8, 14 },
    {  -3,  -6,   5,  6, 12,  7 }, {  -8,  -6,   1, 16, 12,  7 }, {  -8,  -8,   1, 16, 16,  7 }, {  -6,  -8,   1, 12, 16,  7 },
    {  -6,  -8,   5, 12,  6,  7 }, {  -6,  -8,   1, 12, 16,  7 }, {  -8,  -8,   1, 16, 16,  7 }, {  -8,  -6,   1, 16, 12,  7 },
    {  -8,  -6,   5,  6, 12,  7 }, {  -8,  -6,   1, 16, 12,  7 }, {  -8,  -8,   1, 16, 16,  7 }, {  -6,  -8,   1, 12, 16,  7 },
    {  -6,  -3,   5, 12,  6,  7 }, {  -6,  -8,   1, 12, 16,  7 }, {  -8,  -8,   1, 16, 16,  7 }, {  -8,  -6,   1, 16, 12,  7 },
    {   0,  -6,   5,  3, 12,  5 }, {  -6, -13,   5,  3, 12,  5 }, { -13,  -6,   5, 12,  5,  5 }, {  -6,   0,   5, 12,  3,  5 },
    {   0,  -6,  -8, 10, 12, 15 }, {   0,  -6,  -8, 10, 12, 15 }, {   0,  -6,  -8, 10, 12, 15 }, {  -6, -12,  -8,  6,  6, 15 },
    { -12, -12,  -8,  6,  6, 15 }, { -12, -12,  -8,  6,  6, 15 }, { -12, -12,  -8,  6,  6, 15 }, { -12, -12,  -8,  6,  6, 15 },
    { -12, -12,  -8,  6,  6, 15 }, { -12,  -6,  -8,  6,  6, 15 }, { -12,  -6,  -8, 10, 12, 15 }, {  -6,   0,  -8, 12, 10, 15 },
    {  -6,   0,  -8, 12, 10, 15 }, {  -6,   0,  -8, 12, 10, 15 }, {  -6,   0,  -8, 12, 10, 15 }, {  -6,   0,  -8, 12, 10, 15 },
    {  -3,  -6,  -4,  6, 12,  5 }, {  -6, -13,  -4,  3, 20,  5 }, { -13,  -6,  -4,  6,  5,  5 }, {  -6,  -3,  -4, 12, 20,  5 },
    {  -3,  -6,   1,  6, 12,  6 }, {  -6, -13, -10,  3,  6,  6 }, { -13,  -6, -10,  6,  5,  6 }, {  -6,  -3,   1, 12,  6,  6 },
    { -10,  -6, -15, 20, 12,  7 }, {  -6, -10, -15, 12, 20,  7 }, { -10,  -6, -15, 20, 12,  7 }, {  -6, -10, -15, 12, 20,  7 },
    { -10,  -6, -15, 20, 12,  7 }, {  -6, -10, -15, 12, 20,  7 }, { -10,  -6, -15, 20, 12,  7 }, {  -6, -10, -15, 12, 20,  7 },
    { -10,  -6, -15, 20, 12,  7 }, {  -6, -10, -15, 12, 20,  7 }, { -10,  -6, -15, 20, 12,  7 }, {  -6, -10, -15, 12, 20,  7 },
    { -10,  -6, -15, 20, 12,  7 }, {  -6, -10, -15, 12, 20,  7 }, { -10,  -6, -15, 20, 12,  7 }, {  -6, -10, -15, 12, 20,  7 },
    {  -8,  -8,   1, 16, 16, 14 }, {  -8,  -8,   1, 16, 16, 14 }, {  -8,  -8,   1, 16, 16, 14 }, {  -8,  -8,   1, 16, 16, 14 },
    {  -8,  -8,   1, 16, 16, 14 }, { -32, -32,  24,  4,  4, 14 }, {  -8,  -8,   1, 16, 16, 14 }, {   8,   8,   1,  8,  8, 14 },
    {  -8,  -6,   1, 16, 12, 14 }, {  -7,  -6,   1, 14, 12, 14 }, {  -6,  -6,   1, 12, 12, 14 }, {  -6,  -7,   1, 12, 14, 14 },
    {  -6,  -8,   1, 12, 16, 14 }, {  -6,  -7,   1, 12, 14, 14 }, {  -6,  -6,   1, 12, 12, 14 }, {  -7,  -6,   1, 14, 12, 14 },
    {  -8,  -6,   1, 16, 12, 14 }, {  -7,  -6,   1, 14, 12, 14 }, {  -6,  -6,   1, 12, 12, 14 }, {  -6,  -7,   1, 12, 14, 14 },
    {  -6,  -8,   1, 12, 16, 14 }, {  -6,  -7,   1, 12, 14, 14 }, {  -6,  -6,   1, 12, 12, 14 }, {  -7,  -6,   1, 14, 12, 14 },
    {  -6,  -6, -12, 12, 12,  2 }, {  -6,  -6, -12, 12, 12,  2 }, {  -6,  -6,  10, 12, 12, 14 }, {  -6,  -6,  10, 12, 12, 14 },
    {  -6,  -6, -12, 12, 12,  2 }, {  -6,  -6, -12, 12, 12,  2 }, {  -6,  -6,  10, 12, 12, 14 }, {  -6,  -6,  10, 12, 12, 14 },
    {  -6,  -6, -12, 12, 12,  2 }, {  -6,  -6, -12, 12, 12,  2 }, {  -6,  -6,  10, 12, 12, 14 }, {  -6,  -6,  10, 12, 12, 14 },
    {  -6,  -6, -12, 12, 12,  2 }, {  -6,  -6, -12, 12, 12,  2 }, {  -6,  -6,  10, 12, 12, 14 }, {  -6,  -6,  10, 12, 12, 14 },
    {  -6,  -6, -12, 12, 12,  2 }, {  -6,  -6, -12, 12, 12,  2 }, {  -6,  -6, -12, 12, 12,  2 }, {  -6,  -6, -12, 12, 12,  2 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6, -23, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6, -23, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    { -10,  -6, -23, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    { -10,  -6, -23, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    },{ // 0x0098E56C:
    {  -3,  -6,   1,  6, 12, 14 }, {  -3,  -5,   1,  6, 10, 14 }, {  -5,  -5,   1, 10, 10, 14 }, {  -5,  -3,   1, 10,  6, 14 },
    {  -6,  -3,   1, 12,  6, 14 }, {  -5,  -3,   1, 10,  6, 14 }, {  -5,  -5,   1, 10, 10, 14 }, {  -3,  -5,   1,  6, 10, 14 },
    {  -3,  -6,   1,  6, 12, 14 }, {  -3,  -5,   1,  6, 10, 14 }, {  -5,  -5,   1, 10, 10, 14 }, {  -5,  -3,   1, 10,  6, 14 },
    {  -6,  -3,   1, 12,  6, 14 }, {  -5,  -3,   1, 10,  6, 14 }, {  -5,  -5,   1, 10, 10, 14 }, {  -3,  -5,   1,  6, 10, 14 },
    {  -3,  -6,   1,  6, 12, 14 }, {  -3,  -6,   1,  6, 12, 14 }, {  -8,  -8,   1, 16, 16, 14 }, {  -6,  -3,   1, 12,  6, 14 },
    {  -6,  -3,   2, 12,  6, 14 }, {  -6,  -3,   1, 12,  6, 14 }, {  -8,  -8,   1, 16, 16, 14 }, {  -3,  -6,   1,  6, 12, 14 },
    {  -3,  -6,   2,  6, 12, 14 }, {  -3,  -6,   1,  6, 12, 14 }, {  -8,  -8,   1, 16, 16, 14 }, {  -6,  -3,   1, 12,  6, 14 },
    {  -6,  -3,   1, 12,  6, 14 }, {  -6,  -3,   1, 12,  6, 14 }, {  -8,  -8,   1, 16, 16, 14 }, {  -3,  -6,   1,  6, 12, 14 },
    {  -3,  -6,   1,  6, 12,  7 }, {  -8,  -8,   1, 16, 16,  7 }, {  -6,  -3,   1, 12,  6,  7 }, {  -8,  -8,   1, 16, 16,  7 },
    {  -3,  -6,   1,  6, 12,  7 }, {  -8,  -8,   1, 16, 16,  7 }, {  -6,  -3,   1, 12,  6,  7 }, {   0,   0,   1,  8,  8,  7 },
    {  -3,  -6,   5,  6, 12,  7 }, {  -8,  -6,   1, 16, 12,  7 }, {  -8,  -8,   1, 16, 16,  7 }, {  -6,  -8,   1, 12, 16,  7 },
    {  -6,  -3,   5, 12,  6,  7 }, {  -6,  -8,   1, 12, 16,  7 }, {  -8,  -8,   1, 16, 16,  7 }, {  -8,  -6,   1, 16, 12,  7 },
    {  -3,  -6,   5,  6, 12,  7 }, {  -8,  -6,   1, 16, 12,  7 }, {  -8,  -8,   1, 16, 16,  7 }, {  -6,  -8,   1, 12, 16,  7 },
    {  -6,  -3,   5, 12,  6,  7 }, {  -6,  -8,   1, 12, 16,  7 }, {  -8,  -8,   1, 16, 16,  7 }, {  -8,  -6,   1, 16, 12,  7 },
    {   0,  -6,   5,  3, 12,  5 }, {  -6, -13,   5,  3, 12,  5 }, { -13,  -6,   5, 12,  5,  5 }, {  -6,   0,   5, 12,  3,  5 },
    {   0,  -6,  -4, 10, 12,  7 }, {   0,  -6,  -4, 10, 12,  7 }, {   0,  -6,  -4, 10, 12,  7 }, {  -6, -12,  -4, 12, 10,  7 },
    {  -6, -12,  -4, 12, 10,  7 }, {  -6, -12,  -4, 12, 10,  7 }, {  -6, -12,  -4, 12, 10,  7 }, {  -6, -12,  -4, 12, 10,  7 },
    { -12,  -6,  -4, 10, 12,  7 }, { -12,  -6,  -4, 10, 12,  7 }, { -12,  -6,  -4, 10, 12,  7 }, {  -6,   0,  -4, 12, 10,  7 },
    {  -6,   0,  -4, 12, 10,  7 }, {  -6,   0,  -4, 12, 10,  7 }, {  -6,   0,  -4, 12, 10,  7 }, {  -6,   0,  -4, 12, 10,  7 },
    {  -3,  -6,  -4,  6, 12,  5 }, {  -6, -13,  -4,  3, 20,  5 }, { -13,  -6,  -4,  6,  5,  5 }, {  -6,  -3,  -4, 12, 20,  5 },
    {  -3,  -6,   1,  6, 12,  6 }, {  -6, -13, -10,  3,  6,  6 }, { -13,  -6, -10,  6,  5,  6 }, {  -6,  -3,   1, 12,  6,  6 },
    {  -3,  -6, -15,  6, 12,  6 }, {  -6,  -3, -15, 12,  6,  6 }, {  -3,  -6, -15,  6, 12,  6 }, {  -6,  -3, -15, 12,  6,  6 },
    {  -3,  -6, -15,  6, 12,  6 }, {  -6,  -3, -15, 12,  6,  6 }, {  -3,  -6, -15,  6, 12,  6 }, {  -6,  -3, -15, 12,  6,  6 },
    {  -3,  -6, -15,  6, 12,  6 }, {  -6,  -3, -15, 12,  6,  6 }, {  -3,  -6, -15,  6, 12,  6 }, {  -6,  -3, -15, 12,  6,  6 },
    {  -3,  -6, -15,  6, 12,  6 }, {  -6,  -3, -15, 12,  6,  6 }, {  -3,  -6, -15,  6, 12,  6 }, {  -6,  -3, -15, 12,  6,  6 },
    {  -8,  -8,   1, 16, 16, 14 }, {  -8,  -8,   1, 16, 16, 14 }, {  -8,  -8,   1, 16, 16, 14 }, {  -8,  -8,   1, 16, 16, 14 },
    {  -8,  -8,   1, 16, 16, 14 }, { -32, -32,  24,  4,  4, 14 }, {  -8,  -8,   1, 16, 16, 14 }, {   8,   8,   1,  8,  8, 14 },
    {  -3,  -6,   1,  6, 12, 14 }, {  -3,  -5,   1,  6, 10, 14 }, {  -5,  -5,   1, 10, 10, 14 }, {  -5,  -3,   1, 10,  6, 14 },
    {  -6,  -3,   1, 12,  6, 14 }, {  -5,  -3,   1, 10,  6, 14 }, {  -5,  -5,   1, 10, 10, 14 }, {  -3,  -5,   1,  6, 10, 14 },
    {  -3,  -6,   1,  6, 12, 14 }, {  -3,  -5,   1,  6, 10, 14 }, {  -5,  -5,   1, 10, 10, 14 }, {  -5,  -3,   1, 10,  6, 14 },
    {  -6,  -3,   1, 12,  6, 14 }, {  -5,  -3,   1, 10,  6, 14 }, {  -5,  -5,   1, 10, 10, 14 }, {  -3,  -5,   1,  6, 10, 14 },
    {  -6,  -6, -12, 12, 12,  2 }, {  -6,  -6, -12, 12, 12,  2 }, {  -6,  -6,  10, 12, 12, 14 }, {  -6,  -6,  10, 12, 12, 14 },
    {  -6,  -6, -12, 12, 12,  2 }, {  -6,  -6, -12, 12, 12,  2 }, {  -6,  -6,  10, 12, 12, 14 }, {  -6,  -6,  10, 12, 12, 14 },
    {  -6,  -6, -12, 12, 12,  2 }, {  -6,  -6, -12, 12, 12,  2 }, {  -6,  -6,  10, 12, 12, 14 }, {  -6,  -6,  10, 12, 12, 14 },
    {  -6,  -6, -12, 12, 12,  2 }, {  -6,  -6, -12, 12, 12,  2 }, {  -6,  -6,  10, 12, 12, 14 }, {  -6,  -6,  10, 12, 12, 14 },
    {  -6,  -6, -12, 12, 12,  2 }, {  -6,  -6, -12, 12, 12,  2 }, {  -6,  -6, -12, 12, 12,  2 }, {  -6,  -6, -12, 12, 12,  2 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6, -23, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6, -23, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    { -10,  -6, -23, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    { -10,  -6, -23, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    },{ // 0x00990CFC:
    { -12,  -6,   1, 24, 12, 14 }, { -11,  -6,   1, 22, 12, 14 }, { -10, -10,   1, 20, 20, 14 }, {  -6, -11,   1, 12, 22, 14 },
    {  -6, -12,   1, 12, 24, 14 }, {  -6, -11,   1, 12, 22, 14 }, { -10, -10,   1, 20, 20, 14 }, { -11,  -6,   1, 22, 12, 14 },
    { -12,  -6,   1, 24, 12, 14 }, { -11,  -6,   1, 22, 12, 14 }, { -10, -10,   1, 20, 20, 14 }, {  -6, -11,   1, 12, 22, 14 },
    {  -6, -12,   1, 12, 24, 14 }, {  -6, -11,   1, 12, 22, 14 }, { -10, -10,   1, 20, 20, 14 }, { -11,  -6,   1, 22, 12, 14 },
    { -11,  -6,   1, 22, 12, 14 }, { -11,  -6,   1, 22, 12, 14 }, {  -8,  -8,   1, 16, 16, 14 }, {  -6, -11,   1, 12, 22, 14 },
    {  -6, -11,   3, 12, 22, 14 }, {  -6, -11,   1, 12, 22, 14 }, {  -8,  -8,   1, 16, 16, 14 }, { -11,  -6,   1, 22, 12, 14 },
    { -11,  -6,   3, 22, 12, 14 }, { -11,  -6,   1, 22, 12, 14 }, {  -8,  -8,   1, 16, 16, 14 }, {  -6, -11,   1, 12, 22, 14 },
    {  -6, -11,   1, 12, 22, 14 }, {  -6, -11,   1, 12, 22, 14 }, {  -8,  -8,   1, 16, 16, 14 }, { -11,  -6,   1, 22, 12, 14 },
    { -12,  -6,   1, 24, 12, 14 }, {  -8,  -8,   1, 16, 16, 14 }, {  -6, -12,   1, 12, 24, 14 }, {  -8,  -8,   1, 16, 16, 14 },
    { -18,  -6,   1, 24, 12, 14 }, {  -8,  -8,   1, 16, 16, 14 }, {  -6, -14,   1, 12, 24, 14 }, {   0,   0,   1,  8,  8, 14 },
    { -12,  -6,   5, 24, 12, 14 }, { -11,  -6,   1, 22, 12, 14 }, {  -8,  -8,   1, 16, 16, 14 }, {  -6, -11,   1, 12, 22, 14 },
    {  -6, -12,   5, 12, 24, 14 }, {  -6, -11,   1, 12, 22, 14 }, {  -8,  -8,   1, 16, 16, 14 }, { -11,  -6,   1, 22, 12, 14 },
    { -18,  -6,   5, 24, 12, 14 }, { -11,  -6,   1, 22, 12, 14 }, {  -8,  -8,   1, 16, 16, 14 }, {  -6, -11,   1, 12, 22, 14 },
    {  -6, -14,   5, 12, 24, 14 }, {  -6, -11,   1, 12, 22, 14 }, {  -8,  -8,   1, 16, 16, 14 }, { -11,  -6,   1, 22, 12, 14 },
    {   0,  -6, -10, 14, 12, 24 }, {  -6, -17, -10,  3, 14, 24 }, { -17,  -6, -10, 14,  5, 24 }, {  -6,   0, -10, 12, 14, 24 },
    {   0,  -6,  -7, 10, 12, 14 }, {   0,  -6,  -7, 10, 12, 14 }, {   0,  -6,  -7, 10, 12, 14 }, {   0,  -6,  -7, 10, 12, 14 },
    {  -6, -12,  -7, 12, 10, 14 }, {  -6, -12,  -7, 12, 10, 14 }, {  -6, -12,  -7, 12, 10, 14 }, {  -6, -12,  -7, 12, 10, 14 },
    { -12,  -6,  -7, 10, 12, 14 }, { -12,  -6,  -7, 10, 12, 14 }, { -12,  -6,  -7, 10, 12, 14 }, { -12,  -6,  -7, 10, 12, 14 },
    {  -6,   0,  -7, 12, 10, 14 }, {  -6,   0,  -7, 12, 10, 14 }, {  -6,   0,  -7, 12, 10, 14 }, {  -6,   0,  -7, 12, 10, 14 },
    { -14,  -6, -10, 28, 12, 24 }, {  -6, -17, -10,  3, 28, 24 }, { -17,  -6, -10, 28,  5, 24 }, {  -6, -14, -10, 12, 28, 24 },
    { -14,  -6,   1, 28, 12, 14 }, {  -6, -17, -10,  3, 28, 24 }, { -17,  -6, -10, 28,  5, 24 }, {  -6, -14,   1, 12, 28, 14 },
    { -14,  -6, -15, 28, 12,  7 }, {  -6, -14, -15, 12, 28,  7 }, { -14,  -6, -15, 28, 12,  7 }, {  -6, -14, -15, 12, 28,  7 },
    { -14,  -6, -15, 28, 12,  7 }, {  -6, -14, -15, 12, 28,  7 }, { -14,  -6, -15, 28, 12,  7 }, {  -6, -14, -15, 12, 28,  7 },
    { -14,  -6, -15, 28, 12,  7 }, {  -6, -14, -15, 12, 28,  7 }, { -14,  -6, -15, 28, 12,  7 }, {  -6, -14, -15, 12, 28,  7 },
    { -14,  -6, -15, 28, 12,  7 }, {  -6, -14, -15, 12, 28,  7 }, { -14,  -6, -15, 28, 12,  7 }, {  -6, -14, -15, 12, 28,  7 },
    { -10, -10,   1, 20, 20, 14 }, { -10, -10,   1, 20, 20, 14 }, { -10, -10,   1, 20, 20, 14 }, { -10, -10,   1, 20, 20, 14 },
    {  -8,  -8,   1, 16, 16, 14 }, {  -8,  -8,   1, 16, 16, 14 }, {  -8,  -8,   1, 16, 16, 14 }, {   8,   8,   1,  8,  8, 14 },
    { -12,  -6,   1, 24, 12, 14 }, { -11,  -6,   1, 22, 12, 14 }, { -10, -10,   1, 20, 20, 14 }, {  -6, -11,   1, 12, 22, 14 },
    {  -6, -12,   1, 12, 24, 14 }, {  -6, -11,   1, 12, 22, 14 }, { -10, -10,   1, 20, 20, 14 }, { -11,  -6,   1, 22, 12, 14 },
    { -12,  -6,   1, 24, 12, 14 }, { -11,  -6,   1, 22, 12, 14 }, { -10, -10,   1, 20, 20, 14 }, {  -6, -11,   1, 12, 22, 14 },
    {  -6, -12,   1, 12, 24, 14 }, {  -6, -11,   1, 12, 22, 14 }, { -10, -10,   1, 20, 20, 14 }, { -11,  -6,   1, 22, 12, 14 },
    {  -6,  -6, -12, 12, 12,  2 }, {  -6,  -6, -12, 12, 12,  2 }, {  -6,  -6,  10, 12, 12, 14 }, {  -6,  -6,  10, 12, 12, 14 },
    {  -6,  -6, -12, 12, 12,  2 }, {  -6,  -6, -12, 12, 12,  2 }, {  -6,  -6,  10, 12, 12, 14 }, {  -6,  -6,  10, 12, 12, 14 },
    {  -6,  -6, -12, 12, 12,  2 }, {  -6,  -6, -12, 12, 12,  2 }, {  -6,  -6,  10, 12, 12, 14 }, {  -6,  -6,  10, 12, 12, 14 },
    {  -6,  -6, -12, 12, 12,  2 }, {  -6,  -6, -12, 12, 12,  2 }, {  -6,  -6,  10, 12, 12, 14 }, {  -6,  -6,  10, 12, 12, 14 },
    {  -6,  -6, -12, 12, 12,  2 }, {  -6,  -6, -12, 12, 12,  2 }, {  -6,  -6, -12, 12, 12,  2 }, {  -6,  -6, -12, 12, 12,  2 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6, -23, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6, -23, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    { -10,  -6, -23, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    { -10,  -6, -23, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    },{ // 0x0098FA6C:
    { -18,  -6,   1, 36, 12, 11 }, { -17,  -6,   1, 34, 12, 11 }, { -15, -15,   1, 30, 30, 11 }, {  -6, -17,   1, 12, 34, 11 },
    {  -6, -18,   1, 12, 36, 11 }, {  -6, -17,   1, 12, 34, 11 }, { -15, -15,   1, 30, 30, 11 }, { -17,  -6,   1, 34, 12, 11 },
    { -18,  -6,   1, 36, 12, 11 }, { -17,  -6,   1, 34, 12, 11 }, { -15, -15,   1, 30, 30, 11 }, {  -6, -17,   1, 12, 34, 11 },
    {  -6, -18,   1, 12, 36, 11 }, {  -6, -17,   1, 12, 34, 11 }, { -15, -15,   1, 30, 30, 11 }, { -17,  -6,   1, 34, 12, 11 },
    { -14,  -6,   1, 28, 12, 14 }, { -11,  -6,   1, 22, 12, 14 }, {  -8,  -8,   1, 16, 16, 14 }, {  -6, -11,   1, 12, 22, 14 },
    {  -6, -14,   3, 12, 28, 14 }, {  -6, -11,   1, 12, 22, 14 }, {  -8,  -8,   1, 16, 16, 14 }, { -11,  -6,   1, 22, 12, 14 },
    { -14,  -6,   3, 28, 12, 14 }, { -11,  -6,   1, 22, 12, 14 }, {  -8,  -8,   1, 16, 16, 14 }, {  -6, -11,   1, 12, 22, 14 },
    {  -6, -14,   1, 12, 28, 14 }, {  -6, -11,   1, 12, 22, 14 }, {  -8,  -8,   1, 16, 16, 14 }, { -11,  -6,   1, 22, 12, 14 },
    { -14,  -6,   1, 28, 12, 14 }, {  -8,  -8,   1, 16, 16, 14 }, {  -6, -14,   1, 12, 28, 14 }, {  -8,  -8,   1, 16, 16, 14 },
    { -14,  -6,   1, 28, 12, 14 }, {  -8,  -8,   1, 16, 16, 14 }, {  -6, -14,   1, 12, 28, 14 }, {   0,   0,   1,  8,  8, 14 },
    {  -3,  -6,   5,  6, 12,  7 }, {  -8,  -6,   1, 16, 12,  7 }, {  -8,  -8,   1, 16, 16,  7 }, {  -6,  -8,   1, 12, 16,  7 },
    {  -6,  -8,   5, 12,  6,  7 }, {  -6,  -8,   1, 12, 16,  7 }, {  -8,  -8,   1, 16, 16,  7 }, {  -8,  -6,   1, 16, 12,  7 },
    {  -8,  -6,   5,  6, 12,  7 }, {  -8,  -6,   1, 16, 12,  7 }, {  -8,  -8,   1, 16, 16,  7 }, {  -6,  -8,   1, 12, 16,  7 },
    {  -6,  -3,   5, 12,  6,  7 }, {  -6,  -8,   1, 12, 16,  7 }, {  -8,  -8,   1, 16, 16,  7 }, {  -8,  -6,   1, 16, 12,  7 },
    {   0,  -6,   5,  3, 12,  5 }, {  -6, -13,   5,  3, 12,  5 }, { -13,  -6,   5, 12,  5,  5 }, {  -6,   0,   5, 12,  3,  5 },
    {   0,  -6,  -8, 10, 12, 15 }, {   0,  -6,  -8, 10, 12, 15 }, {   0,  -6,  -8, 10, 12, 15 }, {  -6, -12,  -8, 12, 10, 15 },
    {  -6, -12,  -8, 12, 10, 15 }, {  -6, -12,  -8, 12, 10, 15 }, {  -6, -12,  -8, 12, 10, 15 }, {  -6, -12,  -8, 12, 10, 15 },
    { -12,  -6,  -8, 10, 12, 15 }, { -12,  -6,  -8, 10, 12, 15 }, { -12,  -6,  -8, 10, 12, 15 }, {  -6,   0,  -8, 12, 10, 15 },
    {  -6,   0,  -8, 12, 10, 15 }, {  -6,   0,  -8, 12, 10, 15 }, {  -6,   0,  -8, 12, 10, 15 }, {  -6,   0,  -8, 12, 10, 15 },
    {  -3,  -6,  -4,  6, 12,  5 }, {  -6, -13,  -4,  3, 20,  5 }, { -13,  -6,  -4,  6,  5,  5 }, {  -6,  -3,  -4, 12, 20,  5 },
    {  -3,  -6,   1,  6, 12,  6 }, {  -6, -13, -10,  3,  6,  6 }, { -13,  -6, -10,  6,  5,  6 }, {  -6,  -3,   1, 12,  6,  6 },
    { -14,  -6, -15, 28, 12,  7 }, {  -6, -14, -15, 12, 28,  7 }, { -14,  -6, -15, 28, 12,  7 }, {  -6, -14, -15, 12, 28,  7 },
    { -14,  -6, -15, 28, 12,  7 }, {  -6, -14, -15, 12, 28,  7 }, { -14,  -6, -15, 28, 12,  7 }, {  -6, -14, -15, 12, 28,  7 },
    { -14,  -6, -15, 28, 12,  7 }, {  -6, -14, -15, 12, 28,  7 }, { -14,  -6, -15, 28, 12,  7 }, {  -6, -14, -15, 12, 28,  7 },
    { -14,  -6, -15, 28, 12,  7 }, {  -6, -14, -15, 12, 28,  7 }, { -14,  -6, -15, 28, 12,  7 }, {  -6, -14, -15, 12, 28,  7 },
    { -10, -10,   1, 20, 20, 14 }, { -10, -10,   1, 20, 20, 14 }, { -10, -10,   1, 20, 20, 14 }, { -10, -10,   1, 20, 20, 14 },
    {  -8,  -8,   1, 16, 16, 14 }, { -32, -32,  24,  4,  4, 14 }, {  -8,  -8,   1, 16, 16, 14 }, {   8,   8,   1,  8,  8, 14 },
    { -14,  -6,   1, 28, 12, 14 }, { -12,  -6,   1, 24, 12, 14 }, { -10, -10,   1, 20, 20, 14 }, {  -6, -12,   1, 12, 24, 14 },
    {  -6, -14,   1, 12, 28, 14 }, {  -6, -12,   1, 12, 24, 14 }, { -10, -10,   1, 20, 20, 14 }, { -12,  -6,   1, 24, 12, 14 },
    { -14,  -6,   1, 28, 12, 14 }, { -12,  -6,   1, 24, 12, 14 }, { -10, -10,   1, 20, 20, 14 }, {  -6, -12,   1, 12, 24, 14 },
    {  -6, -14,   1, 12, 28, 14 }, {  -6, -12,   1, 12, 24, 14 }, { -10, -10,   1, 20, 20, 14 }, { -12,  -6,   1, 24, 12, 14 },
    {  -6,  -6, -12, 12, 12,  2 }, {  -6,  -6, -12, 12, 12,  2 }, {  -6,  -6,  10, 12, 12, 14 }, {  -6,  -6,  10, 12, 12, 14 },
    {  -6,  -6, -12, 12, 12,  2 }, {  -6,  -6, -12, 12, 12,  2 }, {  -6,  -6,  10, 12, 12, 14 }, {  -6,  -6,  10, 12, 12, 14 },
    {  -6,  -6, -12, 12, 12,  2 }, {  -6,  -6, -12, 12, 12,  2 }, {  -6,  -6,  10, 12, 12, 14 }, {  -6,  -6,  10, 12, 12, 14 },
    {  -6,  -6, -12, 12, 12,  2 }, {  -6,  -6, -12, 12, 12,  2 }, {  -6,  -6,  10, 12, 12, 14 }, {  -6,  -6,  10, 12, 12, 14 },
    {  -6,  -6, -12, 12, 12,  2 }, {  -6,  -6, -12, 12, 12,  2 }, {  -6,  -6, -12, 12, 12,  2 }, {  -6,  -6, -12, 12, 12,  2 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6, -23, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6, -23, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    { -10,  -6, -23, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    { -10,  -6, -23, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    },{ // 0x0099177C:
    { -16,  -6,   1, 32, 12,  8 }, {  -5,  -5,   1, 10, 10,  8 }, {  -4,  -4,   1,  8,  8,  8 }, {  -5,  -5,   1, 10, 10,  8 },
    {  -6, -16,   1, 12, 32,  8 }, {  -5,  -5,   1, 10, 10,  8 }, {  -4,  -4,   1,  8,  8,  8 }, {  -5,  -5,   1, 10, 10,  8 },
    { -16,  -6,   1, 32, 12,  8 }, {  -5,  -5,   1, 10, 10,  8 }, {  -4,  -4,   1,  8,  8,  8 }, {  -5,  -5,   1, 10, 10,  8 },
    {  -6, -16,   1, 12, 32,  8 }, {  -5,  -5,   1, 10, 10,  8 }, {  -4,  -4,   1,  8,  8,  8 }, {  -5,  -5,   1, 10, 10,  8 },
    { -13,  -6,   1, 26, 12, 14 }, { -11,  -6,   1, 22, 12, 14 }, {  -8,  -8,   1, 16, 16, 14 }, {  -6, -11,   1, 12, 22, 14 },
    {  -6, -13,   3, 12, 26, 14 }, {  -6, -11,   1, 12, 22, 14 }, {  -8,  -8,   1, 16, 16, 14 }, { -11,  -6,   1, 22, 12, 14 },
    { -13,  -6,   3, 26, 12, 14 }, { -11,  -6,   1, 22, 12, 14 }, {  -8,  -8,   1, 16, 16, 14 }, {  -6, -11,   1, 12, 22, 14 },
    {  -6, -13,   1, 12, 26, 14 }, {  -6, -11,   1, 12, 22, 14 }, {  -8,  -8,   1, 16, 16, 14 }, { -11,  -6,   1, 22, 12, 14 },
    { -14,  -6,   1, 28, 12, 14 }, {  -8,  -8,   1, 16, 16, 14 }, {  -6, -14,   1, 12, 28, 14 }, {  -8,  -8,   1, 16, 16, 14 },
    { -18,  -6,   1, 28, 12, 14 }, {  -8,  -8,   1, 16, 16, 14 }, {  -6, -14,   1, 12, 28, 14 }, {   0,   0,   1,  8,  8, 14 },
    { -14,  -6,   5, 28, 12, 14 }, { -11,  -6,   1, 22, 12, 14 }, {  -8,  -8,   1, 16, 16, 14 }, {  -6, -11,   1, 12, 22, 14 },
    {  -6, -14,   5, 12, 28, 14 }, {  -6, -11,   1, 12, 22, 14 }, {  -8,  -8,   1, 16, 16, 14 }, { -11,  -6,   1, 22, 12, 14 },
    { -18,  -6,   5, 28, 12, 14 }, { -11,  -6,   1, 22, 12, 14 }, {  -8,  -8,   1, 16, 16, 14 }, {  -6, -11,   1, 12, 22, 14 },
    {  -6, -14,   5, 12, 28, 14 }, {  -6, -11,   1, 12, 22, 14 }, {  -8,  -8,   1, 16, 16, 14 }, { -11,  -6,   1, 22, 12, 14 },
    { -14,  -6, -10, 28, 12, 24 }, {  -6, -17, -10,  3, 28, 24 }, { -17,  -6, -10, 28,  5, 24 }, {  -6, -14, -10, 12, 28, 24 },
    {   0,  -6,  -7, 10, 12, 14 }, {   0,  -6,  -7, 10, 12, 14 }, {   0,  -6,  -7, 10, 12, 14 }, {   0,  -6,  -7, 10, 12, 14 },
    {  -6, -12,  -7, 12, 10, 14 }, {  -6, -12,  -7, 12, 10, 14 }, {  -6, -12,  -7, 12, 10, 14 }, {  -6, -12,  -7, 12, 10, 14 },
    { -12,  -6,  -7, 10, 12, 14 }, { -12,  -6,  -7, 10, 12, 14 }, { -12,  -6,  -7, 10, 12, 14 }, { -12,  -6,  -7, 10, 12, 14 },
    {  -6,   0,  -7, 12, 10, 14 }, {  -6,   0,  -7, 12, 10, 14 }, {  -6,   0,  -7, 12, 10, 14 }, {  -6,   0,  -7, 12, 10, 14 },
    { -14,  -6, -10, 28, 12, 24 }, {  -6, -17, -10,  3, 28, 24 }, { -17,  -6, -10, 28,  5, 24 }, {  -6, -14, -10, 12, 28, 24 },
    { -14,  -6,   1, 28, 12, 14 }, {  -6, -17, -10,  3, 28, 24 }, { -17,  -6, -10, 28,  5, 24 }, {  -6, -14,   1, 12, 28, 14 },
    { -14,  -6, -15, 28, 12,  7 }, {  -6, -14, -15, 12, 28,  7 }, { -14,  -6, -15, 28, 12,  7 }, {  -6, -14, -15, 12, 28,  7 },
    { -14,  -6, -15, 28, 12,  7 }, {  -6, -14, -15, 12, 28,  7 }, { -14,  -6, -15, 28, 12,  7 }, {  -6, -14, -15, 12, 28,  7 },
    { -14,  -6, -15, 28, 12,  7 }, {  -6, -14, -15, 12, 28,  7 }, { -14,  -6, -15, 28, 12,  7 }, {  -6, -14, -15, 12, 28,  7 },
    { -16,  -6,   1, 32, 12,  8 }, {  -6, -16,   1, 12, 32,  8 }, { -16,  -6,   1, 32, 12,  8 }, {  -6, -16,   1, 12, 32,  8 },
    { -10, -10,   1, 20, 20, 14 }, { -10, -10,   1, 20, 20, 14 }, { -10, -10,   1, 20, 20, 14 }, { -10, -10,   1, 20, 20, 14 },
    {  -8,  -8,   1, 16, 16, 14 }, {  -8,  -8,   1, 16, 16, 14 }, {  -8,  -8,   1, 16, 16, 14 }, {   8,   8,   1,  8,  8, 14 },
    { -16,  -6,   1, 32, 12,  8 }, {  -5,  -5,   1, 10, 10,  8 }, {  -4,  -4,   1,  8,  8,  8 }, {  -5,  -5,   1, 10, 10,  8 },
    {  -6, -16,   1, 12, 32,  8 }, {  -5,  -5,   1, 10, 10,  8 }, {  -4,  -4,   1,  8,  8,  8 }, {  -5,  -5,   1, 10, 10,  8 },
    { -16,  -6,   1, 32, 12,  8 }, {  -5,  -5,   1, 10, 10,  8 }, {  -4,  -4,   1,  8,  8,  8 }, {  -5,  -5,   1, 10, 10,  8 },
    {  -6, -16,   1, 12, 32,  8 }, {  -5,  -5,   1, 10, 10,  8 }, {  -4,  -4,   1,  8,  8,  8 }, {  -5,  -5,   1, 10, 10,  8 },
    { -16,  -6,   1, 32, 12,  8 }, {  -6, -16,   1, 12, 32,  8 }, { -16,  -6,   1, 32, 12,  8 }, {  -6, -16,   1, 12, 32,  8 },
    { -16,  -6,   1, 32, 12,  8 }, {  -6, -16,   1, 12, 32,  8 }, { -16,  -6,   1, 32, 12,  8 }, {  -6, -16,   1, 12, 32,  8 },
    { -16,  -6,   1, 32, 12,  8 }, {  -6, -16,   1, 12, 32,  8 }, { -16,  -6,   1, 32, 12,  8 }, {  -6, -16,   1, 12, 32,  8 },
    { -16,  -6,   1, 32, 12,  8 }, {  -6, -16,   1, 12, 32,  8 }, { -16,  -6,   1, 32, 12,  8 }, {  -6, -16,   1, 12, 32,  8 },
    { -16,  -6,   1, 32, 12,  8 }, {  -6, -16,   1, 12, 32,  8 }, { -16,  -6,   1, 32, 12,  8 }, {  -6, -16,   1, 12, 32,  8 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6, -23, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6, -23, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    { -10,  -6,   1, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6,   1, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    { -10,  -6, -23, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    { -10,  -6, -23, 20, 12, 14 }, {  -6, -10,   1, 12, 20, 14 }, { -10,  -6, -23, 20, 12, 14 }, {  -6, -10, -23, 12, 20, 14 },
    },{ // 0x009900FC:
    { -16,  -6,   1, 32, 12, 14 }, { -12,  -6,   1, 24, 12, 14 }, { -10, -10,   1, 20, 20, 14 }, {  -6, -12,   1, 12, 24, 14 },
    {  -6, -16,   1, 12, 32, 14 }, {  -6, -12,   1, 12, 24, 14 }, { -10, -10,   1, 20, 20, 14 }, { -12,  -6,   1, 24, 12, 14 },
    { -16,  -6,   1, 32, 12, 14 }, { -12,  -6,   1, 24, 12, 14 }, { -10, -10,   1, 20, 20, 14 }, {  -6, -12,   1, 12, 24, 14 },
    {  -6, -16,   1, 12, 32, 14 }, {  -6, -12,   1, 12, 24, 14 }, { -10, -10,   1, 20, 20, 14 }, { -12,  -6,   1, 24, 12, 14 },
    { -14,  -6,   5, 28, 12,  9 }, { -11,  -6,   1, 22, 12, 14 }, {  -8,  -8,   1, 16, 16, 14 }, {  -6, -11,   1, 12, 22, 14 },
    {  -6, -14,   5, 12, 28,  9 }, {  -6, -11,   1, 12, 22, 14 }, {  -8,  -8,   1, 16, 16, 14 }, { -11,  -6,   1, 22, 12, 14 },
    { -14,  -6,   5, 28, 12,  9 }, { -11,  -6,   1, 22, 12, 14 }, {  -8,  -8,   1, 16, 16, 14 }, {  -6, -11,   1, 12, 22, 14 },
    {  -6, -14,   5, 12, 28,  9 }, {  -6, -11,   1, 12, 22, 14 }, {  -8,  -8,   1, 16, 16, 14 }, { -11,  -6,   1, 22, 12, 14 },
    {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 },
    {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 },
    {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 },
    {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 },
    {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 },
    {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 },
    {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 },
    {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 },
    {  -8,  -6, -28, 16, 12, 14 }, {  -8,  -6, -28, 16, 12, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -6,  -8, -28, 12, 16, 14 },
    {  -6,  -8, -28, 12, 16, 14 }, {  -6,  -8, -28, 12, 16, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -8,  -6, -28, 16, 12, 14 },
    {  -8,  -6, -28, 16, 12, 14 }, {  -8,  -6, -28, 16, 12, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -6,  -8, -28, 12, 16, 14 },
    {  -6,  -8, -28, 12, 16, 14 }, {  -6,  -8, -28, 12, 16, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -8,  -6, -28, 16, 12, 14 },
    {  -6,  -6, -28, 12, 12, 14 }, {  -9,  -6, -28, 18, 12, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -6,  -9, -28, 12, 18, 14 },
    {  -6,  -6, -28, 12, 12, 14 }, {  -6,  -9, -28, 12, 18, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -9,  -6, -28, 18, 12, 14 },
    {  -6,  -6, -28, 12, 12, 14 }, {  -9,  -6, -28, 18, 12, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -6,  -9, -28, 12, 18, 14 },
    {  -6,  -6, -28, 12, 12, 14 }, {  -6,  -9, -28, 12, 18, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -9,  -6, -28, 18, 12, 14 },
    { -10,  -6, -28, 16, 12, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -6, -11, -33,  3, 16, 19 }, {  -8,  -8, -28, 16, 16, 14 },
    { -11,  -6, -33, 16,  5, 19 }, {  -8,  -8, -28, 16, 16, 14 }, {  -6, -10, -28, 12, 16, 14 }, {  -8,  -8, -28, 16, 16, 14 },
    { -10,  -6, -28, 16, 12, 14 }, {  -6,  -6, -28, 12, 12, 14 }, {  -6,  -6, -28, 12, 12, 14 }, {  -6,  -6, -28, 12, 12, 14 },
    {  -3, -10, -39,  3, 16, 27 }, {  -6,  -6, -28, 12, 12, 14 }, {  -6,  -6, -28, 12, 12, 14 }, {  -6,  -6, -28, 12, 12, 14 },
    { -10,  -6, -39, 16,  3, 27 }, {  -6,  -6, -28, 12, 12, 14 }, {  -6,  -6, -28, 12, 12, 14 }, {  -6,  -6, -28, 12, 12, 14 },
    {  -6, -10, -28, 12, 16, 14 }, {  -6,  -6, -28, 12, 12, 14 }, {  -6,  -6, -28, 12, 12, 14 }, {  -6,  -6, -28, 12, 12, 14 },
    { -10,  -6, -28, 20, 12, 14 }, {  -2,  -3, -39,  3, 10, 29 }, {  -3,  -2, -39, 10,  3, 29 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6, -49,  5, 12, 34 }, { -10,  -6, -49,  5, 12, 34 }, { -10,  -6, -49,  5, 12, 34 }, { -10,  -6, -49,  5, 12, 34 },
    {  -2,   4, -39,  3, 10, 29 }, {  -2,   4, -39,  3, 10, 29 }, {  -2,   4, -39,  3, 10, 29 }, {  -2,   4, -39,  3, 10, 29 },
    { -13,  -6, -39, 20,  5, 24 }, { -13,  -6, -39, 20,  5, 24 }, { -13,  -6, -39, 20,  5, 24 }, { -13,  -6, -39, 20,  5, 24 },
    {  -6, -15, -28, 12, 10, 14 }, {  -6, -15, -28, 12, 10, 14 }, {  -6, -15, -28, 12, 10, 14 }, {  -6, -15, -28, 12, 10, 14 },
    { -10,  -6, -39,  4, 12, 24 }, {  -2,   4, -10,  3, 10, 24 }, {  -3,  -6, -39, 10,  5, 24 }, {  -6, -15, -10, 12, 10, 14 },
    { -10,  -6, -28,  3, 12, 14 }, {  -6,   0, -10,  3, 10, 24 }, {  -3,  -6, -28, 10,  5, 14 }, {  -6, -15, -10, 12, 10, 14 },
    { -10,  -6, -44, 20, 12,  7 }, {  -6,   0,  -5, 12, 10,  7 }, {   0,  -6, -34, 10, 12,  7 }, {  -6, -15,  -5, 12, 10,  7 },
    { -10,  -6,   5, 20, 12,  7 }, {  -6,   0,   5, 12, 10,  7 }, {   0,  -6,   5, 10, 12,  7 }, {  -6, -10,   5, 12, 10,  7 },
    { -10,  -6,   5, 20, 12,  7 }, {  -6, -10,   5, 12, 20,  7 }, { -10,  -6,   5, 20, 12,  7 }, {  -6, -10,   5, 12, 20,  7 },
    { -10,  -6,   5, 20, 12,  7 }, {  -6, -10,   5, 12, 20,  7 }, { -10,  -6,   5, 20, 12,  7 }, {  -6, -10,   5, 12, 20,  7 },
    {  -8,  -8, -28, 16, 16, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -8,  -8, -28, 16, 16, 14 },
    {  -8,  -8, -28, 16, 16, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -8,  -8, -28, 16, 16, 14 },
    {  -8,  -6,  11, 16, 12,  4 }, {  -8,  -6,  11, 16, 12,  4 }, {  -8,  -8,  11, 16, 16,  4 }, {  -6,  -8,  11, 12, 16,  4 },
    {  -6,  -8,  11, 12, 16,  4 }, {  -6,  -8, -28, 12, 16, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -8,  -6, -28, 16, 12, 14 },
    {  -8,  -6, -28, 16, 12, 14 }, {  -8,  -6, -28, 16, 12, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -6,  -8, -28, 12, 16, 14 },
    {  -6,  -8, -28, 12, 16, 14 }, {  -6,  -8, -28, 12, 16, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -8,  -6, -28, 16, 12, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 },
    { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 }, { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 }, { -10,  -6,  11, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 }, { -10,  -6,  11, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6, -28, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 },
    },{ // 0x009901BC:
    {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 },
    {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 },
    {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 },
    {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 },
    {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 },
    {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 },
    {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 },
    {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 },
    {  -8,  -6, -28, 16, 12, 14 }, {  -8,  -6, -28, 16, 12, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -6,  -8, -28, 12, 16, 14 },
    {  -6,  -8, -28, 12, 16, 14 }, {  -6,  -8, -28, 12, 16, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -8,  -6, -28, 16, 12, 14 },
    {  -8,  -6, -28, 16, 12, 14 }, {  -8,  -6, -28, 16, 12, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -6,  -8, -28, 12, 16, 14 },
    {  -6,  -8, -28, 12, 16, 14 }, {  -6,  -8, -28, 12, 16, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -8,  -6, -28, 16, 12, 14 },
    {  -6,  -6, -28, 12, 12, 14 }, {  -9,  -6, -28, 18, 12, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -6,  -9, -28, 12, 18, 14 },
    {  -6,  -6, -28, 12, 12, 14 }, {  -6,  -9, -28, 12, 18, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -9,  -6, -28, 18, 12, 14 },
    {  -6,  -6, -28, 12, 12, 14 }, {  -9,  -6, -28, 18, 12, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -6,  -9, -28, 12, 18, 14 },
    {  -6,  -6, -28, 12, 12, 14 }, {  -6,  -9, -28, 12, 18, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -9,  -6, -28, 18, 12, 14 },
    { -10,  -6, -28, 16, 12, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -6, -11, -33,  3, 16, 19 }, {  -8,  -8, -28, 16, 16, 14 },
    { -11,  -6, -33, 16,  5, 19 }, {  -8,  -8, -28, 16, 16, 14 }, {  -6, -10, -28, 12, 16, 14 }, {  -8,  -8, -28, 16, 16, 14 },
    { -10,  -6, -28, 16, 12, 14 }, {  -6,  -6, -28, 12, 12, 14 }, {  -6,  -6, -28, 12, 12, 14 }, {  -6,  -6, -28, 12, 12, 14 },
    {  -3, -10, -39,  3, 16, 27 }, {  -6,  -6, -28, 12, 12, 14 }, {  -6,  -6, -28, 12, 12, 14 }, {  -6,  -6, -28, 12, 12, 14 },
    { -10,  -6, -39, 16,  3, 27 }, {  -6,  -6, -28, 12, 12, 14 }, {  -6,  -6, -28, 12, 12, 14 }, {  -6,  -6, -28, 12, 12, 14 },
    {  -6, -10, -28, 12, 16, 14 }, {  -6,  -6, -28, 12, 12, 14 }, {  -6,  -6, -28, 12, 12, 14 }, {  -6,  -6, -28, 12, 12, 14 },
    { -10,  -6, -28, 20, 12, 14 }, {  -2,  -3, -39,  3, 10, 29 }, {  -3,  -2, -39, 10,  3, 29 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6, -49,  5, 12, 34 }, { -10,  -6, -49,  5, 12, 34 }, { -10,  -6, -49,  5, 12, 34 }, { -10,  -6, -49,  5, 12, 34 },
    {  -2,   4, -39,  3, 10, 29 }, {  -2,   4, -39,  3, 10, 29 }, {  -2,   4, -39,  3, 10, 29 }, {  -2,   4, -39,  3, 10, 29 },
    { -13,  -6, -39, 20,  5, 24 }, { -13,  -6, -39, 20,  5, 24 }, { -13,  -6, -39, 20,  5, 24 }, { -13,  -6, -39, 20,  5, 24 },
    {  -6, -15, -28, 12, 10, 14 }, {  -6, -15, -28, 12, 10, 14 }, {  -6, -15, -28, 12, 10, 14 }, {  -6, -15, -28, 12, 10, 14 },
    { -10,  -6, -39,  4, 12, 24 }, {  -2,   4, -10,  3, 10, 24 }, {  -3,  -6, -39, 10,  5, 24 }, {  -6, -15, -10, 12, 10, 14 },
    { -10,  -6, -28,  3, 12, 14 }, {  -6,   0, -10,  3, 10, 24 }, {  -3,  -6, -28, 10,  5, 14 }, {  -6, -15, -10, 12, 10, 14 },
    { -10,  -6, -44, 20, 12,  7 }, {  -6,   0,  -5, 12, 10,  7 }, {   0,  -6, -34, 10, 12,  7 }, {  -6, -15,  -5, 12, 10,  7 },
    { -10,  -6,   5, 20, 12,  7 }, {  -6,   0,   5, 12, 10,  7 }, {   0,  -6,   5, 10, 12,  7 }, {  -6, -10,   5, 12, 10,  7 },
    { -10,  -6,   5, 20, 12,  7 }, {  -6, -10,   5, 12, 20,  7 }, { -10,  -6,   5, 20, 12,  7 }, {  -6, -10,   5, 12, 20,  7 },
    { -10,  -6,   5, 20, 12,  7 }, {  -6, -10,   5, 12, 20,  7 }, { -10,  -6,   5, 20, 12,  7 }, {  -6, -10,   5, 12, 20,  7 },
    {  -8,  -8, -28, 16, 16, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -8,  -8, -28, 16, 16, 14 },
    {  -8,  -8, -28, 16, 16, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -8,  -8, -28, 16, 16, 14 },
    {  -8,  -6,  11, 16, 12,  4 }, {  -8,  -6,  11, 16, 12,  4 }, {  -8,  -8,  11, 16, 16,  4 }, {  -6,  -8,  11, 12, 16,  4 },
    {  -6,  -8,  11, 12, 16,  4 }, {  -6,  -8, -28, 12, 16, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -8,  -6, -28, 16, 12, 14 },
    {  -8,  -6, -28, 16, 12, 14 }, {  -8,  -6, -28, 16, 12, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -6,  -8, -28, 12, 16, 14 },
    {  -6,  -8, -28, 12, 16, 14 }, {  -6,  -8, -28, 12, 16, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -8,  -6, -28, 16, 12, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 },
    { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 }, { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 }, { -10,  -6,  11, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 }, { -10,  -6,  11, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6, -28, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 },
    { -10,  -6, -28, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 },
    { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 }, { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 }, { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 }, { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6, -28, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 },
    { -10,  -6,  11, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6, -28, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 },
    { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 }, { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6, -28, 20, 12, 14 }, {  -6, -10,  11, 12, 20, 14 }, { -10,  -6, -28, 20, 12, 14 }, {  -6, -10, -28, 12, 20, 14 },
    },{ // 0x0098FFAC:
    {  -3,  -6,  -1,  6, 12,  1 }, {  -3,  -5,  -1,  6, 10,  1 }, {  -5,  -5,  -1, 10, 10,  1 }, {  -5,  -3,  -1, 10,  6,  1 },
    {  -6,  -3,  -1, 12,  6,  1 }, {  -5,  -3,  -1, 10,  6,  1 }, {  -5,  -5,  -1, 10, 10,  1 }, {  -3,  -5,  -1,  6, 10,  1 },
    {  -3,  -6,  -1,  6, 12,  1 }, {  -3,  -5,  -1,  6, 10,  1 }, {  -5,  -5,  -1, 10, 10,  1 }, {  -5,  -3,  -1, 10,  6,  1 },
    {  -6,  -3,  -1, 12,  6,  1 }, {  -5,  -3,  -1, 10,  6,  1 }, {  -5,  -5,  -1, 10, 10,  1 }, {  -3,  -5,  -1,  6, 10,  1 },
    {  -3,  -6,  -8,  6, 12,  1 }, {  -3,  -6,  -1,  6, 12,  1 }, {  -8,  -8,  -1, 16, 16,  1 }, {  -6,  -3,  -1, 12,  6,  1 },
    {  -6,  -3,   0, 12,  6,  1 }, {  -6,  -3,  -1, 12,  6,  1 }, {  -8,  -8,  -1, 16, 16,  1 }, {  -3,  -6,  -1,  6, 12,  1 },
    {  -3,  -6,   0,  6, 12,  1 }, {  -3,  -6,  -1,  6, 12,  1 }, {  -8,  -8,  -1, 16, 16,  1 }, {  -6,  -3,  -1, 12,  6,  1 },
    {  -6,  -3,  -8, 12,  6,  1 }, {  -6,  -3,  -1, 12,  6,  1 }, {  -8,  -8,  -1, 16, 16,  1 }, {  -3,  -6,  -1,  6, 12,  1 },
    {  -3,  -6, -10,  6, 12,  1 }, {  -8,  -8,  -2, 16, 16,  1 }, {  -6,  -3,  -2, 12,  6,  1 }, {  -8,  -8,  -2, 16, 16,  1 },
    {  -3,  -6,  -2,  6, 12,  1 }, {  -8,  -8,  -2, 16, 16,  1 }, {  -6,  -3, -10, 12,  6,  1 }, {   0,   0,  -2,  8,  8,  1 },
    {  -3,  -6, -12,  6, 12,  1 }, {  -8,  -6,  -6, 16, 12,  1 }, {  -8,  -8,  -6, 16, 16,  1 }, {  -6,  -8,  -6, 12, 16,  1 },
    {  -6,  -3,  -2, 12,  6,  1 }, {  -6,  -8,  -6, 12, 16,  1 }, {  -8,  -8,  -6, 16, 16,  1 }, {  -8,  -6,  -6, 16, 12,  1 },
    {  -3,  -6,  -2,  6, 12,  1 }, {  -8,  -6,  -6, 16, 12,  1 }, {  -8,  -8,  -6, 16, 16,  1 }, {  -6,  -8,  -6, 12, 16,  1 },
    {  -6,  -3, -12, 12,  6,  1 }, {  -6,  -8,  -6, 12, 16,  1 }, {  -8,  -8,  -6, 16, 16,  1 }, {  -8,  -6,  -6, 16, 12,  1 },
    { -16,  -6,   1, 32, 12, 14 }, { -12,  -6,   1, 24, 12, 14 }, { -10, -10,   1, 20, 20, 14 }, {  -6, -12,   1, 12, 24, 14 },
    {  -6, -16,   1, 12, 32, 14 }, {  -6, -12,   1, 12, 24, 14 }, { -10, -10,   1, 20, 20, 14 }, { -12,  -6,   1, 24, 12, 14 },
    { -16,  -6,   1, 32, 12, 14 }, { -12,  -6,   1, 24, 12, 14 }, { -10, -10,   1, 20, 20, 14 }, {  -6, -12,   1, 12, 24, 14 },
    {  -6, -16,   1, 12, 32, 14 }, {  -6, -12,   1, 12, 24, 14 }, { -10, -10,   1, 20, 20, 14 }, { -12,  -6,   1, 24, 12, 14 },
    { -14,  -6,   5, 28, 12,  9 }, { -11,  -6,   1, 22, 12, 14 }, {  -8,  -8,   1, 16, 16, 14 }, {  -6, -11,   1, 12, 22, 14 },
    {  -6, -14,   5, 12, 28,  9 }, {  -6, -11,   1, 12, 22, 14 }, {  -8,  -8,   1, 16, 16, 14 }, { -11,  -6,   1, 22, 12, 14 },
    { -14,  -6,   5, 28, 12,  9 }, { -11,  -6,   1, 22, 12, 14 }, {  -8,  -8,   1, 16, 16, 14 }, {  -6, -11,   1, 12, 22, 14 },
    {  -6, -14,   5, 12, 28,  9 }, {  -6, -11,   1, 12, 22, 14 }, {  -8,  -8,   1, 16, 16, 14 }, { -11,  -6,   1, 22, 12, 14 },
    {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 },
    {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 },
    {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 },
    {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 },
    {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 },
    {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 },
    {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 },
    {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 }, {  -6,  -6,  -4, 12, 12,  1 },
    {  -8,  -6, -28, 16, 12, 14 }, {  -8,  -6, -28, 16, 12, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -6,  -8, -28, 12, 16, 14 },
    {  -6,  -8, -28, 12, 16, 14 }, {  -6,  -8, -28, 12, 16, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -8,  -6, -28, 16, 12, 14 },
    {  -8,  -6, -28, 16, 12, 14 }, {  -8,  -6, -28, 16, 12, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -6,  -8, -28, 12, 16, 14 },
    {  -6,  -8, -28, 12, 16, 14 }, {  -6,  -8, -28, 12, 16, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -8,  -6, -28, 16, 12, 14 },
    {  -6,  -6, -28, 12, 12, 14 }, {  -9,  -6, -28, 18, 12, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -6,  -9, -28, 12, 18, 14 },
    {  -6,  -6, -28, 12, 12, 14 }, {  -6,  -9, -28, 12, 18, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -9,  -6, -28, 18, 12, 14 },
    {  -6,  -6, -28, 12, 12, 14 }, {  -9,  -6, -28, 18, 12, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -6,  -9, -28, 12, 18, 14 },
    {  -6,  -6, -28, 12, 12, 14 }, {  -6,  -9, -28, 12, 18, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -9,  -6, -28, 18, 12, 14 },
    { -10,  -6, -28, 16, 12, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -6, -11, -33,  3, 16, 19 }, {  -8,  -8, -28, 16, 16, 14 },
    { -11,  -6, -33, 16,  5, 19 }, {  -8,  -8, -28, 16, 16, 14 }, {  -6, -10, -28, 12, 16, 14 }, {  -8,  -8, -28, 16, 16, 14 },
    { -10,  -6, -28, 16, 12, 14 }, {  -6,  -6, -28, 12, 12, 14 }, {  -6,  -6, -28, 12, 12, 14 }, {  -6,  -6, -28, 12, 12, 14 },
    {  -3, -10, -39,  3, 16, 27 }, {  -6,  -6, -28, 12, 12, 14 }, {  -6,  -6, -28, 12, 12, 14 }, {  -6,  -6, -28, 12, 12, 14 },
    { -10,  -6, -39, 16,  3, 27 }, {  -6,  -6, -28, 12, 12, 14 }, {  -6,  -6, -28, 12, 12, 14 }, {  -6,  -6, -28, 12, 12, 14 },
    {  -6, -10, -28, 12, 16, 14 }, {  -6,  -6, -28, 12, 12, 14 }, {  -6,  -6, -28, 12, 12, 14 }, {  -6,  -6, -28, 12, 12, 14 },
    { -10,  -6, -28, 20, 12, 14 }, {  -2,  -3, -39,  3, 10, 29 }, {  -3,  -2, -39, 10,  3, 29 }, {  -6, -10, -28, 12, 20, 14 },
    { -10,  -6, -49,  5, 12, 34 }, { -10,  -6, -49,  5, 12, 34 }, { -10,  -6, -49,  5, 12, 34 }, { -10,  -6, -49,  5, 12, 34 },
    {  -2,   4, -39,  3, 10, 29 }, {  -2,   4, -39,  3, 10, 29 }, {  -2,   4, -39,  3, 10, 29 }, {  -2,   4, -39,  3, 10, 29 },
    { -13,  -6, -39, 20,  5, 24 }, { -13,  -6, -39, 20,  5, 24 }, { -13,  -6, -39, 20,  5, 24 }, { -13,  -6, -39, 20,  5, 24 },
    {  -6, -15, -28, 12, 10, 14 }, {  -6, -15, -28, 12, 10, 14 }, {  -6, -15, -28, 12, 10, 14 }, {  -6, -15, -28, 12, 10, 14 },
    { -10,  -6, -39,  4, 12, 24 }, {  -2,   4, -10,  3, 10, 24 }, {  -3,  -6, -39, 10,  5, 24 }, {  -6, -15, -10, 12, 10, 14 },
    { -10,  -6, -28,  3, 12, 14 }, {  -6,   0, -10,  3, 10, 24 }, {  -3,  -6, -28, 10,  5, 14 }, {  -6, -15, -10, 12, 10, 14 },
    { -10,  -6, -44, 20, 12,  7 }, {  -6,   0,  -5, 12, 10,  7 }, {   0,  -6, -34, 10, 12,  7 }, {  -6, -15,  -5, 12, 10,  7 },
    { -10,  -6,   5, 20, 12,  7 }, {  -6,   0,   5, 12, 10,  7 }, {   0,  -6,   5, 10, 12,  7 }, {  -6, -10,   5, 12, 10,  7 },
    { -10,  -6,   5, 20, 12,  7 }, {  -6, -10,   5, 12, 20,  7 }, { -10,  -6,   5, 20, 12,  7 }, {  -6, -10,   5, 12, 20,  7 },
    { -10,  -6,   5, 20, 12,  7 }, {  -6, -10,   5, 12, 20,  7 }, { -10,  -6,   5, 20, 12,  7 }, {  -6, -10,   5, 12, 20,  7 },
    {  -8,  -8, -28, 16, 16, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -8,  -8, -28, 16, 16, 14 }, {  -8,  -8, -28, 16, 16, 14 },
    },{ // 0x0099273C:
    {  -6,  -6,   0, 12, 12, 31 }, {  -6,  -6,   0, 12, 12, 31 }, {  -6,  -6,   0, 12, 12, 31 }, {  -6,  -6,   0, 12, 12, 31 },
    {  -6,  -6,   0, 12, 12, 31 }, {  -6,  -6,   0, 12, 12, 31 }, {  -6,  -6,   0, 12, 12, 31 }, {  -6,  -6,   0, 12, 12, 31 },
    {  -6,  -6,   0, 12, 12, 31 }, {  -6,  -6,   0, 12, 12, 31 }, {  -6,  -6,   0, 12, 12, 31 }, {  -6,  -6,   0, 12, 12, 31 },
    {  -6,  -6,   0, 12, 12, 31 }, {  -6,  -6,   0, 12, 12, 31 }, {  -6,  -6,   0, 12, 12, 31 }, {  -6,  -6,   0, 12, 12, 31 },
    }
};

// 6D5214
static void vehicle_sprite_paint(rct_vehicle *vehicle, sint32 ebx, sint32 ecx, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{

    sint32 baseImage_id = ebx;
    vehicle_boundbox bb = VehicleBoundboxes[vehicleEntry->draw_order][ecx];
    if (vehicleEntry->flags & VEHICLE_ENTRY_FLAG_14) {
        baseImage_id += (vehicle->var_BA / 8) & 31;
    }
    if (vehicleEntry->flags & VEHICLE_ENTRY_FLAG_7) {
        baseImage_id += vehicle->var_C5;
    }
    uint32 rotation = get_current_rotation();
    sint32 image_id = baseImage_id | (vehicle->colours.body_colour << 19) | (vehicle->colours.trim_colour << 24) | IMAGE_TYPE_REMAP_2_PLUS;
    paint_struct* ps = sub_98197C(image_id, 0, 0, bb.length_x, bb.length_y, bb.length_z, z, bb.offset_x, bb.offset_y, bb.offset_z + z, rotation);
    if (ps != NULL) {
        ps->tertiary_colour = vehicle->colours_extended;
    }
    rct_drawpixelinfo* dpi = unk_140E9A8;
    if (dpi->zoom_level < 2 && vehicle->num_peeps > 0 && vehicleEntry->no_seating_rows > 0) {
        baseImage_id += vehicleEntry->no_vehicle_images;
        for (sint32 i = 0; i < 8; i++){
            if (vehicle->num_peeps > (i * 2) && vehicleEntry->no_seating_rows > i) {
                image_id = baseImage_id | SPRITE_ID_PALETTE_COLOUR_2(vehicle->peep_tshirt_colours[i * 2], vehicle->peep_tshirt_colours[(i * 2) + 1]);
                if (i == 0 && vehicleEntry->flags & VEHICLE_ENTRY_FLAG_24) {
                    image_id += (vehicleEntry->no_vehicle_images * vehicle->var_C5);
                }
                sub_98199C(image_id, 0, 0,  bb.length_x, bb.length_y, bb.length_z, z, bb.offset_x, bb.offset_y, bb.offset_z + z, rotation);
                baseImage_id += vehicleEntry->no_vehicle_images;
            }
        }
    }
    vehicle_visual_splash_effect(z, vehicle, vehicleEntry);
}

// 6D520E
static void vehicle_sprite_paint_6D520E(rct_vehicle *vehicle, sint32 ebx, sint32 ecx, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    vehicle_sprite_paint(vehicle, ebx + vehicle->var_4A, ecx, z, vehicleEntry);
}

// 6D51EB
static void vehicle_sprite_paint_6D51EB(rct_vehicle *vehicle, sint32 ebx, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    sint32 ecx = ebx / 2;
    if (vehicleEntry->flags & VEHICLE_ENTRY_FLAG_11) {
        ebx = ebx / 2;
    }
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_15) {
        ebx = ebx / 8;
    }
    ebx = (ebx * vehicleEntry->var_16) + vehicle->var_4A + vehicleEntry->base_image_id;
    vehicle_sprite_paint(vehicle, ebx, ecx, z, vehicleEntry);
}

// 6D51DE
static void vehicle_sprite_paint_6D51DE(rct_vehicle *vehicle, sint32 ebx, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    if (vehicle->restraints_position < 64) {
        vehicle_sprite_paint_6D51EB(vehicle, ebx, z, vehicleEntry);
        return;
    }
    if (!(vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_RESTRAINT_ANIMATION)) {
        vehicle_sprite_paint_6D51EB(vehicle, ebx, z, vehicleEntry);
        return;
    }
    if (ebx & 7) {
        vehicle_sprite_paint_6D51EB(vehicle, ebx, z, vehicleEntry);
        return;
    }
    sint32 ecx = ebx / 2;
    ebx = ebx / 8;
    ebx += ((vehicle->restraints_position - 64) / 64) * 4;
    ebx *= vehicleEntry->var_16;
    ebx += vehicleEntry->var_1C;
    vehicle_sprite_paint(vehicle, ebx, ecx, z, vehicleEntry);
}

// 6D51DE
static void vehicle_sprite_0_0(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    vehicle_sprite_paint_6D51DE(vehicle, imageDirection, z, vehicleEntry);
}

// 6D4EE7
static void vehicle_sprite_0_1(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_FLAT_BANKED) {
        sint32 ecx = imageDirection / 2;
        sint32 ebx = ((imageDirection / 4) * vehicleEntry->var_16) + vehicleEntry->var_30;
        vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
    } else {
        vehicle_sprite_paint_6D51DE(vehicle, imageDirection, z, vehicleEntry);
    }
}

// 6D4F34
static void vehicle_sprite_0_2(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_FLAT_BANKED) {
        sint32 ecx = (imageDirection / 2) + 108;
        sint32 ebx = ((imageDirection + 16) * vehicleEntry->var_16) + vehicleEntry->var_30;
        vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
    } else {
        vehicle_sprite_paint_6D51DE(vehicle, imageDirection, z, vehicleEntry);
    }
}

// 6D4F0C
static void vehicle_sprite_0_3(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_FLAT_BANKED) {
        sint32 ecx = imageDirection / 2;
        sint32 ebx = (((imageDirection / 4) + 8) * vehicleEntry->var_16) + vehicleEntry->var_30;
        vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
    } else {
        vehicle_sprite_paint_6D51DE(vehicle, imageDirection, z, vehicleEntry);
    }
}

// 6D4F5C
static void vehicle_sprite_0_4(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_FLAT_BANKED) {
        sint32 ecx = ((imageDirection / 2) ^ 8) + 108;
        sint32 ebx = ((imageDirection + 48) * vehicleEntry->var_16) + vehicleEntry->var_30;
        vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
    } else {
        vehicle_sprite_paint_6D51DE(vehicle, imageDirection, z, vehicleEntry);
    }
}

// 6D4F84
static void vehicle_sprite_0_5(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    if (vehicle->update_flags & VEHICLE_UPDATE_FLAG_USE_INVERTED_SPRITES) {
        vehicleEntry--;
    }
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_INLINE_TWISTS) {
        sint32 ecx = (imageDirection / 8) + 124;
        sint32 ebx = ((imageDirection / 8) * vehicleEntry->var_16) + vehicleEntry->var_34;
        vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
    } else {
        vehicle_sprite_0_2(vehicle, imageDirection, z, vehicleEntry);
    }
}

// 6D4FE4
static void vehicle_sprite_0_6(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    if (vehicle->update_flags & VEHICLE_UPDATE_FLAG_USE_INVERTED_SPRITES) {
        vehicleEntry--;
    }
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_INLINE_TWISTS) {
        sint32 ecx = (imageDirection / 8) + 128;
        sint32 ebx = (((imageDirection / 8) + 8) * vehicleEntry->var_16) + vehicleEntry->var_34;
        vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
    } else {
        vehicle_sprite_0_2(vehicle, imageDirection, z, vehicleEntry);
    }
}

// 6D5055
static void vehicle_sprite_0_7(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    if (vehicle->update_flags & VEHICLE_UPDATE_FLAG_USE_INVERTED_SPRITES) {
        vehicleEntry--;
    }
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_INLINE_TWISTS) {
        sint32 ecx = (imageDirection / 8) + 132;
        sint32 ebx = (((imageDirection / 8) + 16) * vehicleEntry->var_16) + vehicleEntry->var_34;
        vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
    } else {
        vehicle_sprite_0_2(vehicle, imageDirection, z, vehicleEntry);
    }
}

// 6D50C6
static void vehicle_sprite_0_8(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    if (vehicle->update_flags & VEHICLE_UPDATE_FLAG_USE_INVERTED_SPRITES) {
        vehicleEntry--;
    }
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_INLINE_TWISTS) {
        sint32 ecx = (imageDirection / 8) + 136;
        sint32 ebx = (((imageDirection / 8) + 24) * vehicleEntry->var_16) + vehicleEntry->var_34;
        vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
    } else {
        vehicle_sprite_0_2(vehicle, imageDirection, z, vehicleEntry);
    }
}

// 6D5137
static void vehicle_sprite_0_9(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    if (vehicle->update_flags & VEHICLE_UPDATE_FLAG_USE_INVERTED_SPRITES) {
        vehicleEntry--;
    }
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_INLINE_TWISTS) {
        sint32 ecx = (imageDirection / 8) + 140;
        sint32 ebx = (((imageDirection / 8) + 32) * vehicleEntry->var_16) + vehicleEntry->var_34;
        vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
    } else {
        vehicle_sprite_0_2(vehicle, imageDirection, z, vehicleEntry);
    }
}

// 6D4FB1
static void vehicle_sprite_0_10(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    if (vehicle->update_flags & VEHICLE_UPDATE_FLAG_USE_INVERTED_SPRITES) {
        vehicleEntry--;
    }
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_INLINE_TWISTS) {
        sint32 ecx = ((imageDirection / 8) ^ 2) + 124;
        sint32 ebx = (((imageDirection / 8) + 4) * vehicleEntry->var_16) + vehicleEntry->var_34;
        vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
    } else {
        vehicle_sprite_0_4(vehicle, imageDirection, z, vehicleEntry);
    }
}

// 6D501B
static void vehicle_sprite_0_11(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    if (vehicle->update_flags & VEHICLE_UPDATE_FLAG_USE_INVERTED_SPRITES) {
        vehicleEntry--;
    }
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_INLINE_TWISTS) {
        sint32 ecx = ((imageDirection / 8) ^ 2) + 128;
        sint32 ebx = (((imageDirection / 8) + 12) * vehicleEntry->var_16) + vehicleEntry->var_34;
        vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
    } else {
        vehicle_sprite_0_4(vehicle, imageDirection, z, vehicleEntry);
    }
}

// 6D508C
static void vehicle_sprite_0_12(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    if (vehicle->update_flags & VEHICLE_UPDATE_FLAG_USE_INVERTED_SPRITES) {
        vehicleEntry--;
    }
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_INLINE_TWISTS) {
        sint32 ecx = ((imageDirection / 8) ^ 2) + 132;
        sint32 ebx = (((imageDirection / 8) + 20) * vehicleEntry->var_16) + vehicleEntry->var_34;
        vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
    } else {
        vehicle_sprite_0_4(vehicle, imageDirection, z, vehicleEntry);
    }
}

// 6D50FD
static void vehicle_sprite_0_13(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    if (vehicle->update_flags & VEHICLE_UPDATE_FLAG_USE_INVERTED_SPRITES) {
        vehicleEntry--;
    }
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_INLINE_TWISTS) {
        sint32 ecx = ((imageDirection / 8) ^ 2) + 136;
        sint32 ebx = (((imageDirection / 8) + 28) * vehicleEntry->var_16) + vehicleEntry->var_34;
        vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
    } else {
        vehicle_sprite_0_4(vehicle, imageDirection, z, vehicleEntry);
    }
}

// 6D516E
static void vehicle_sprite_0_14(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    if (vehicle->update_flags & VEHICLE_UPDATE_FLAG_USE_INVERTED_SPRITES) {
        vehicleEntry--;
    }
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_INLINE_TWISTS) {
        sint32 ecx = ((imageDirection / 8) ^ 2) + 140;
        sint32 ebx = (((imageDirection / 8) + 36) * vehicleEntry->var_16) + vehicleEntry->var_34;
        vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
    } else {
        vehicle_sprite_0_2(vehicle, imageDirection, z, vehicleEntry);
    }
}

// 6D4EE4
static void vehicle_sprite_0_16(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    vehicleEntry--;
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_FLAT_BANKED) {
        sint32 ecx = imageDirection / 2;
        sint32 ebx = ((imageDirection / 4) * vehicleEntry->var_16) + vehicleEntry->var_30;
        vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
    } else {
        vehicle_sprite_paint_6D51DE(vehicle, imageDirection, z, vehicleEntry);
    }
}

// 6D4F31
static void vehicle_sprite_0_17(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    vehicleEntry--;
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_FLAT_BANKED) {
        sint32 ecx = (imageDirection / 2) + 108;
        sint32 ebx = ((imageDirection + 16) * vehicleEntry->var_16) + vehicleEntry->var_30;
        vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
    } else {
        vehicle_sprite_paint_6D51DE(vehicle, imageDirection, z, vehicleEntry);
    }
}

// 6D4F09
static void vehicle_sprite_0_18(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    vehicleEntry--;
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_FLAT_BANKED) {
        sint32 ecx = imageDirection / 2;
        sint32 ebx = (((imageDirection / 4) + 8) * vehicleEntry->var_16) + vehicleEntry->var_30;
        vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
    } else {
        vehicle_sprite_paint_6D51DE(vehicle, imageDirection, z, vehicleEntry);
    }
}

// 6D4F59
static void vehicle_sprite_0_19(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    vehicleEntry--;
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_FLAT_BANKED) {
        sint32 ecx = ((imageDirection / 2) ^ 8) + 108;
        sint32 ebx = ((imageDirection + 48) * vehicleEntry->var_16) + vehicleEntry->var_30;
        vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
    } else {
        vehicle_sprite_paint_6D51DE(vehicle, imageDirection, z, vehicleEntry);
    }
}

// 6D51D7
static void vehicle_sprite_0(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    // 0x009A3DE4:
    switch (vehicle->bank_rotation) {
        case 0:  vehicle_sprite_0_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 1:  vehicle_sprite_0_1(vehicle, imageDirection, z, vehicleEntry); break;
        case 2:  vehicle_sprite_0_2(vehicle, imageDirection, z, vehicleEntry); break;
        case 3:  vehicle_sprite_0_3(vehicle, imageDirection, z, vehicleEntry); break;
        case 4:  vehicle_sprite_0_4(vehicle, imageDirection, z, vehicleEntry); break;
        case 5:  vehicle_sprite_0_5(vehicle, imageDirection, z, vehicleEntry); break;
        case 6:  vehicle_sprite_0_6(vehicle, imageDirection, z, vehicleEntry); break;
        case 7:  vehicle_sprite_0_7(vehicle, imageDirection, z, vehicleEntry); break;
        case 8:  vehicle_sprite_0_8(vehicle, imageDirection, z, vehicleEntry); break;
        case 9:  vehicle_sprite_0_9(vehicle, imageDirection, z, vehicleEntry); break;
        case 10: vehicle_sprite_0_10(vehicle, imageDirection, z, vehicleEntry); break;
        case 11: vehicle_sprite_0_11(vehicle, imageDirection, z, vehicleEntry); break;
        case 12: vehicle_sprite_0_12(vehicle, imageDirection, z, vehicleEntry); break;
        case 13: vehicle_sprite_0_13(vehicle, imageDirection, z, vehicleEntry); break;
        case 14: vehicle_sprite_0_14(vehicle, imageDirection, z, vehicleEntry); break;
        case 15: vehicle_sprite_0_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 16: vehicle_sprite_0_16(vehicle, imageDirection, z, vehicleEntry); break;
        case 17: vehicle_sprite_0_17(vehicle, imageDirection, z, vehicleEntry); break;
        case 18: vehicle_sprite_0_18(vehicle, imageDirection, z, vehicleEntry); break;
        case 19: vehicle_sprite_0_19(vehicle, imageDirection, z, vehicleEntry); break;
    }
}

// 6D4614
static void vehicle_sprite_1_0(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_GENTLE_SLOPES) {
        sint32 ecx = imageDirection / 2;
        sint32 ebx = ((imageDirection / 8) * vehicleEntry->var_16) + vehicleEntry->var_20;
        vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
    } else {
        vehicle_sprite_0(vehicle, imageDirection, z, vehicleEntry);
    }
}

// 6D4662
static void vehicle_sprite_1_1(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_FLAT_TO_GENTLE_SLOPE_BANKED_TRANSITIONS) {
        sint32 ecx = imageDirection / 2;
        sint32 ebx = (imageDirection * vehicleEntry->var_16) + vehicleEntry->var_38;
        vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
    } else {
        vehicle_sprite_1_0(vehicle, imageDirection, z, vehicleEntry);
    }
}

// 6D46DB
static void vehicle_sprite_1_2(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_FLAT_TO_GENTLE_SLOPE_WHILE_BANKED_TRANSITIONS) {
        sint32 ecx = imageDirection / 2;
        sint32 ebx = ((imageDirection / 8) * vehicleEntry->var_16) + vehicleEntry->var_48;
        vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
    } else {
        vehicle_sprite_1_1(vehicle, imageDirection, z, vehicleEntry);
    }
}

// 6D467D
static void vehicle_sprite_1_3(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_FLAT_TO_GENTLE_SLOPE_BANKED_TRANSITIONS) {
        sint32 ecx = imageDirection / 2;
        sint32 ebx = ((imageDirection + 32) * vehicleEntry->var_16) + vehicleEntry->var_38;
        vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
    } else {
        vehicle_sprite_1_0(vehicle, imageDirection, z, vehicleEntry);
    }
}

// 6D46FD
static void vehicle_sprite_1_4(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_FLAT_TO_GENTLE_SLOPE_WHILE_BANKED_TRANSITIONS) {
        sint32 ecx = imageDirection / 2;
        sint32 ebx = (((imageDirection / 8) + 4) * vehicleEntry->var_16) + vehicleEntry->var_48;
        vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
    } else {
        vehicle_sprite_1_3(vehicle, imageDirection, z, vehicleEntry);
    }
}

// 6D460D
static void vehicle_sprite_1(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    // 0x009A3C04:
    switch (vehicle->bank_rotation) {
        case 0:  vehicle_sprite_1_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 1:  vehicle_sprite_1_1(vehicle, imageDirection, z, vehicleEntry); break;
        case 2:  vehicle_sprite_1_2(vehicle, imageDirection, z, vehicleEntry); break;
        case 3:  vehicle_sprite_1_3(vehicle, imageDirection, z, vehicleEntry); break;
        case 4:  vehicle_sprite_1_4(vehicle, imageDirection, z, vehicleEntry); break;
        case 5:  vehicle_sprite_1_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 6:  vehicle_sprite_1_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 7:  vehicle_sprite_1_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 8:  vehicle_sprite_1_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 9:  vehicle_sprite_1_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 10: vehicle_sprite_1_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 11: vehicle_sprite_1_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 12: vehicle_sprite_1_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 13: vehicle_sprite_1_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 14: vehicle_sprite_1_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 15: vehicle_sprite_1_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 16: vehicle_sprite_1_1(vehicle, imageDirection, z, vehicleEntry); break;
        case 17: vehicle_sprite_1_2(vehicle, imageDirection, z, vehicleEntry); break;
        case 18: vehicle_sprite_1_3(vehicle, imageDirection, z, vehicleEntry); break;
        case 19: vehicle_sprite_1_4(vehicle, imageDirection, z, vehicleEntry); break;
    }
}

// 6D4791
static void vehicle_sprite_2_0(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_GENTLE_SLOPES) {
        if (vehicleEntry->flags & VEHICLE_ENTRY_FLAG_14) {
            sint32 ecx = (imageDirection / 2) + 16;
            sint32 ebx = (((imageDirection / 8) + 8) * vehicleEntry->var_16) + vehicleEntry->var_20;
            vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
        } else {
            sint32 ecx = (imageDirection / 2) + 16;
            sint32 ebx = ((imageDirection + 8) * vehicleEntry->var_16) + vehicleEntry->var_20;
            vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
        }
    } else {
        vehicle_sprite_0(vehicle, imageDirection, z, vehicleEntry);
    }
}

// 6D4833
static void vehicle_sprite_2_1(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_GENTLE_SLOPE_BANKED_TRANSITIONS) {
        sint32 ecx = (imageDirection / 2) + 16;
        sint32 ebx = ((imageDirection / 8) * vehicleEntry->var_16) + vehicleEntry->var_40;
        vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
    } else {
        vehicle_sprite_2_0(vehicle, imageDirection, z, vehicleEntry);
    }
}

// 6D48D6
static void vehicle_sprite_2_2(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_GENTLE_SLOPE_BANKED_TURNS) {
        sint32 ecx = imageDirection / 2;
        if (vehicleEntry->draw_order < 5) {
            ecx += 108;
            sint32 ebx = (imageDirection * vehicleEntry->var_16) + vehicleEntry->var_44;
            vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
        } else {
            ecx += 16;
            sint32 ebx = (imageDirection * vehicleEntry->var_16) + vehicleEntry->var_44;
            vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
        }
    } else {
        vehicle_sprite_2_0(vehicle, imageDirection, z, vehicleEntry);
    }
}

// 6D4858
static void vehicle_sprite_2_3(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_GENTLE_SLOPE_BANKED_TRANSITIONS) {
        sint32 ecx = (imageDirection / 2) + 16;
        sint32 ebx = (((imageDirection / 8) + 4) * vehicleEntry->var_16) + vehicleEntry->var_40;
        vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
    } else {
        vehicle_sprite_2_0(vehicle, imageDirection, z, vehicleEntry);
    }
}

// 6D4910
static void vehicle_sprite_2_4(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_GENTLE_SLOPE_BANKED_TURNS) {
        sint32 ecx = imageDirection / 2;
        if (vehicleEntry->draw_order < 5) {
            ecx = (ecx ^ 8) + 108;
            sint32 ebx = ((imageDirection + 32) * vehicleEntry->var_16) + vehicleEntry->var_44;
            vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
        } else {
            ecx += 16;
            sint32 ebx = ((imageDirection + 32) * vehicleEntry->var_16) + vehicleEntry->var_44;
            vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
        }
    } else {
        vehicle_sprite_2_0(vehicle, imageDirection, z, vehicleEntry);
    }
}

// 6D476C
static void vehicle_sprite_2(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    // 0x009A3CA4:
    switch (vehicle->bank_rotation) {
        case 0:  vehicle_sprite_2_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 1:  vehicle_sprite_2_1(vehicle, imageDirection, z, vehicleEntry); break;
        case 2:  vehicle_sprite_2_2(vehicle, imageDirection, z, vehicleEntry); break;
        case 3:  vehicle_sprite_2_3(vehicle, imageDirection, z, vehicleEntry); break;
        case 4:  vehicle_sprite_2_4(vehicle, imageDirection, z, vehicleEntry); break;
        case 5:  vehicle_sprite_2_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 6:  vehicle_sprite_2_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 7:  vehicle_sprite_2_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 8:  vehicle_sprite_2_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 9:  vehicle_sprite_2_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 10: vehicle_sprite_2_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 11: vehicle_sprite_2_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 12: vehicle_sprite_2_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 13: vehicle_sprite_2_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 14: vehicle_sprite_2_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 15: vehicle_sprite_2_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 16: vehicle_sprite_2_1(vehicle, imageDirection, z, vehicleEntry); break;
        case 17: vehicle_sprite_2_2(vehicle, imageDirection, z, vehicleEntry); break;
        case 18: vehicle_sprite_2_3(vehicle, imageDirection, z, vehicleEntry); break;
        case 19: vehicle_sprite_2_4(vehicle, imageDirection, z, vehicleEntry); break;
    }
}

// 6D49DC
static void vehicle_sprite_3(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    if (!(vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_STEEP_SLOPES)) {
        vehicle_sprite_2(vehicle, imageDirection, z, vehicleEntry);
    } else {
        sint32 ecx = (imageDirection / 4) + 32;
        sint32 ebx = ((imageDirection / 4) * vehicleEntry->var_16) + vehicleEntry->var_24;
        vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
    }
}

// 6D4A31
static void vehicle_sprite_4(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    if (!(vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_STEEP_SLOPES)) {
        vehicle_sprite_2(vehicle, imageDirection, z, vehicleEntry);
    } else {
        sint32 ecx = (imageDirection / 2) + 40;
        sint32 ebx = ((imageDirection + 16) * vehicleEntry->var_16) + vehicleEntry->var_24;
        vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
    }
}

// 6D463D
static void vehicle_sprite_5_0(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_GENTLE_SLOPES) {
        sint32 ecx = imageDirection / 2;
        sint32 ebx = (((imageDirection / 8) + 4) * vehicleEntry->var_16) + vehicleEntry->var_20;
        vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
    } else {
        vehicle_sprite_0(vehicle, imageDirection, z, vehicleEntry);
    }
}

// 6D469B
static void vehicle_sprite_5_1(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_FLAT_TO_GENTLE_SLOPE_BANKED_TRANSITIONS) {
        sint32 ecx = imageDirection / 2;
        sint32 ebx = ((imageDirection + 64) * vehicleEntry->var_16) + vehicleEntry->var_38;
        vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
    } else {
        vehicle_sprite_5_0(vehicle, imageDirection, z, vehicleEntry);
    }
}

// 6D4722
static void vehicle_sprite_5_2(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_FLAT_TO_GENTLE_SLOPE_WHILE_BANKED_TRANSITIONS) {
        sint32 ecx = imageDirection / 2;
        sint32 ebx = (((imageDirection / 8) + 8) * vehicleEntry->var_16) + vehicleEntry->var_48;
        vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
    } else {
        vehicle_sprite_5_1(vehicle, imageDirection, z, vehicleEntry);
    }
}

// 6D46B9
static void vehicle_sprite_5_3(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_FLAT_TO_GENTLE_SLOPE_BANKED_TRANSITIONS) {
        sint32 ecx = imageDirection / 2;
        sint32 ebx = ((imageDirection + 96) * vehicleEntry->var_16) + vehicleEntry->var_38;
        vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
    } else {
        vehicle_sprite_5_0(vehicle, imageDirection, z, vehicleEntry);
    }
}

// 6D4747
static void vehicle_sprite_5_4(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_FLAT_TO_GENTLE_SLOPE_WHILE_BANKED_TRANSITIONS) {
        sint32 ecx = imageDirection / 2;
        sint32 ebx = (((imageDirection / 8) + 12) * vehicleEntry->var_16) + vehicleEntry->var_48;
        vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
    } else {
        vehicle_sprite_5_3(vehicle, imageDirection, z, vehicleEntry);
    }
}

// 6D4636
static void vehicle_sprite_5(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    // 0x009A3C54:
    switch (vehicle->bank_rotation) {
        case 0:  vehicle_sprite_5_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 1:  vehicle_sprite_5_1(vehicle, imageDirection, z, vehicleEntry); break;
        case 2:  vehicle_sprite_5_2(vehicle, imageDirection, z, vehicleEntry); break;
        case 3:  vehicle_sprite_5_3(vehicle, imageDirection, z, vehicleEntry); break;
        case 4:  vehicle_sprite_5_4(vehicle, imageDirection, z, vehicleEntry); break;
        case 5:  vehicle_sprite_5_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 6:  vehicle_sprite_5_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 7:  vehicle_sprite_5_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 8:  vehicle_sprite_5_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 9:  vehicle_sprite_5_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 10: vehicle_sprite_5_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 11: vehicle_sprite_5_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 12: vehicle_sprite_5_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 13: vehicle_sprite_5_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 14: vehicle_sprite_5_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 15: vehicle_sprite_5_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 16: vehicle_sprite_5_1(vehicle, imageDirection, z, vehicleEntry); break;
        case 17: vehicle_sprite_5_2(vehicle, imageDirection, z, vehicleEntry); break;
        case 18: vehicle_sprite_5_3(vehicle, imageDirection, z, vehicleEntry); break;
        case 19: vehicle_sprite_5_4(vehicle, imageDirection, z, vehicleEntry); break;
    }
}

// 6D47E4
static void vehicle_sprite_6_0(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_GENTLE_SLOPES) {
        if (vehicleEntry->flags & VEHICLE_ENTRY_FLAG_14) {
            sint32 ecx = ((imageDirection / 2) ^ 8) + 16;
            sint32 ebx = (((imageDirection / 8) + 12) * vehicleEntry->var_16) + vehicleEntry->var_20;
            vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
        } else {
            sint32 ecx = ((imageDirection / 2) ^ 8) + 16;
            sint32 ebx = ((imageDirection + 40) * vehicleEntry->var_16) + vehicleEntry->var_20;
            vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
        }
    } else {
        vehicle_sprite_0(vehicle, imageDirection, z, vehicleEntry);
    }
}

// 6D4880
static void vehicle_sprite_6_1(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_GENTLE_SLOPE_BANKED_TRANSITIONS) {
        sint32 ecx = ((imageDirection / 2) ^ 8) + 16;
        sint32 ebx = (((imageDirection / 8) + 8) * vehicleEntry->var_16) + vehicleEntry->var_40;
        vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
    } else {
        vehicle_sprite_6_0(vehicle, imageDirection, z, vehicleEntry);
    }
}

// 6D4953
static void vehicle_sprite_6_2(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_GENTLE_SLOPE_BANKED_TURNS) {
        sint32 ecx = imageDirection / 2;
        if (vehicleEntry->draw_order < 5) {
            ecx += 108;
            sint32 ebx = ((imageDirection + 64) * vehicleEntry->var_16) + vehicleEntry->var_44;
            vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
        } else {
            ecx = (ecx ^ 8) + 16;
            sint32 ebx = ((imageDirection + 64) * vehicleEntry->var_16) + vehicleEntry->var_44;
            vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
        }
    } else {
        vehicle_sprite_6_0(vehicle, imageDirection, z, vehicleEntry);
    }
}

// 6D48AB
static void vehicle_sprite_6_3(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_GENTLE_SLOPE_BANKED_TRANSITIONS) {
        sint32 ecx = ((imageDirection / 2) ^ 8) + 16;
        sint32 ebx = (((imageDirection / 8) + 12) * vehicleEntry->var_16) + vehicleEntry->var_40;
        vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
    } else {
        vehicle_sprite_6_0(vehicle, imageDirection, z, vehicleEntry);
    }
}

// 6D4996
static void vehicle_sprite_6_4(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_GENTLE_SLOPE_BANKED_TURNS) {
        sint32 ecx = imageDirection / 2;
        if (vehicleEntry->draw_order < 5) {
            ecx = (ecx ^ 8) + 108;
            sint32 ebx = ((imageDirection + 96) * vehicleEntry->var_16) + vehicleEntry->var_44;
            vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
        } else {
            ecx = (ecx ^ 8) + 16;
            sint32 ebx = ((imageDirection + 96) * vehicleEntry->var_16) + vehicleEntry->var_44;
            vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
        }
    } else {
        vehicle_sprite_6_0(vehicle, imageDirection, z, vehicleEntry);
    }
}

// 6D47DD
static void vehicle_sprite_6(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    // 0x009A3CF4:
    switch (vehicle->bank_rotation) {
        case 0:  vehicle_sprite_6_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 1:  vehicle_sprite_6_1(vehicle, imageDirection, z, vehicleEntry); break;
        case 2:  vehicle_sprite_6_2(vehicle, imageDirection, z, vehicleEntry); break;
        case 3:  vehicle_sprite_6_3(vehicle, imageDirection, z, vehicleEntry); break;
        case 4:  vehicle_sprite_6_4(vehicle, imageDirection, z, vehicleEntry); break;
        case 5:  vehicle_sprite_6_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 6:  vehicle_sprite_6_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 7:  vehicle_sprite_6_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 8:  vehicle_sprite_6_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 9:  vehicle_sprite_6_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 10: vehicle_sprite_6_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 11: vehicle_sprite_6_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 12: vehicle_sprite_6_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 13: vehicle_sprite_6_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 14: vehicle_sprite_6_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 15: vehicle_sprite_6_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 16: vehicle_sprite_6_1(vehicle, imageDirection, z, vehicleEntry); break;
        case 17: vehicle_sprite_6_2(vehicle, imageDirection, z, vehicleEntry); break;
        case 18: vehicle_sprite_6_3(vehicle, imageDirection, z, vehicleEntry); break;
        case 19: vehicle_sprite_6_4(vehicle, imageDirection, z, vehicleEntry); break;
    }
}

// 6D4A05
static void vehicle_sprite_7(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_STEEP_SLOPES) {
        sint32 ecx = ((imageDirection / 4) ^ 4) + 32;
        sint32 ebx = (((imageDirection / 4) + 8) * vehicleEntry->var_16) + vehicleEntry->var_24;
        vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
    } else {
        vehicle_sprite_6(vehicle, imageDirection, z, vehicleEntry);
    }
}

// 6D4A59
static void vehicle_sprite_8(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_STEEP_SLOPES) {
        sint32 ecx = ((imageDirection / 2) ^ 8) + 40;
        sint32 ebx = ((imageDirection + 48) * vehicleEntry->var_16) + vehicleEntry->var_24;
        vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
    } else {
        vehicle_sprite_6(vehicle, imageDirection, z, vehicleEntry);
    }
}

// 6D4A81
static void vehicle_sprite_9(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_VERTICAL_SLOPES) {
        sint32 ecx = (imageDirection / 8) + 56;
        sint32 ebx = ((imageDirection / 8) * vehicleEntry->var_16) + vehicleEntry->var_28;
        vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
    } else {
        vehicle_sprite_4(vehicle, imageDirection, z, vehicleEntry);
    }
}

// 6D4AE8
static void vehicle_sprite_10(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_VERTICAL_SLOPES) {
        sint32 ecx = (imageDirection / 2) + 60;
        sint32 ebx = ((imageDirection + 8) * vehicleEntry->var_16) + vehicleEntry->var_28;
        vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
    } else {
        vehicle_sprite_4(vehicle, imageDirection, z, vehicleEntry);
    }
}

// 6D4B57
static void vehicle_sprite_11(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_VERTICAL_SLOPES) {
        sint32 ecx = (imageDirection / 8) + 76;
        sint32 ebx = (((imageDirection / 8) + 72) * vehicleEntry->var_16) + vehicleEntry->var_28;
        vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
    } else {
        vehicle_sprite_4(vehicle, imageDirection, z, vehicleEntry);
    }
}

// 6D4BB7
static void vehicle_sprite_12(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_VERTICAL_SLOPES) {
        sint32 ecx = (imageDirection / 8) + 80;
        sint32 ebx = (((imageDirection / 8) + 80) * vehicleEntry->var_16) + vehicleEntry->var_28;
        vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
    } else {
        vehicle_sprite_4(vehicle, imageDirection, z, vehicleEntry);
    }
}

// 6D4C17
static void vehicle_sprite_13(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_VERTICAL_SLOPES) {
        sint32 ecx = (imageDirection / 8) + 84;
        sint32 ebx = (((imageDirection / 8) + 88) * vehicleEntry->var_16) + vehicleEntry->var_28;
        vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
    } else {
        vehicle_sprite_4(vehicle, imageDirection, z, vehicleEntry);
    }
}

// 6D4C77
static void vehicle_sprite_14(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_VERTICAL_SLOPES) {
        sint32 ecx = (imageDirection / 8) + 88;
        sint32 ebx = (((imageDirection / 8) + 96) * vehicleEntry->var_16) + vehicleEntry->var_28;
        vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
    } else {
        vehicle_sprite_4(vehicle, imageDirection, z, vehicleEntry);
    }
}

// 6D4CD7
static void vehicle_sprite_15(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_VERTICAL_SLOPES) {
        sint32 ecx = (imageDirection / 8) + 92;
        sint32 ebx = (((imageDirection / 8) + 104) * vehicleEntry->var_16) + vehicleEntry->var_28;
        vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
    } else {
        vehicle_sprite_4(vehicle, imageDirection, z, vehicleEntry);
    }
}

// 6D4D37
static void vehicle_sprite_16(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_VERTICAL_SLOPES) {
        sint32 ecx = (imageDirection / 8) + 96;
        sint32 ebx = (((imageDirection / 8) + 112) * vehicleEntry->var_16) + vehicleEntry->var_28;
        vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
    } else {
        vehicle_sprite_4(vehicle, imageDirection, z, vehicleEntry);
    }
}

// 6D4AA3
static void vehicle_sprite_17(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    if (vehicle->update_flags & VEHICLE_UPDATE_FLAG_USE_INVERTED_SPRITES) {
        if( (vehicle->track_type >> 2) != TRACK_ELEM_90_DEG_DOWN_TO_60_DEG_DOWN &&
            (vehicle->track_type >> 2) != TRACK_ELEM_60_DEG_DOWN_TO_90_DEG_DOWN) {
            vehicleEntry--;
        }
    }
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_VERTICAL_SLOPES) {
        sint32 ecx = ((imageDirection / 8) ^ 2) + 56;
        sint32 ebx = (((imageDirection / 8) + 4) * vehicleEntry->var_16) + vehicleEntry->var_28;
        vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
    } else {
        vehicle_sprite_8(vehicle, imageDirection, z, vehicleEntry);
    }
}

// 6D4B0D
static void vehicle_sprite_18(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    if (vehicle->update_flags & VEHICLE_UPDATE_FLAG_USE_INVERTED_SPRITES) {
        if( (vehicle->track_type >> 2) != TRACK_ELEM_90_DEG_DOWN &&
            (vehicle->track_type >> 2) != TRACK_ELEM_90_DEG_DOWN_TO_60_DEG_DOWN &&
            (vehicle->track_type >> 2) != TRACK_ELEM_60_DEG_DOWN_TO_90_DEG_DOWN) {
            vehicleEntry--;
        }
    }
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_VERTICAL_SLOPES) {
        sint32 ecx = ((imageDirection / 2) ^ 8) + 60;
        sint32 ebx = ((imageDirection + 40) * vehicleEntry->var_16) + vehicleEntry->var_28;
        vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
    } else {
        vehicle_sprite_8(vehicle, imageDirection, z, vehicleEntry);
    }
}

// 6D4B80
static void vehicle_sprite_19(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    if (vehicle->update_flags & VEHICLE_UPDATE_FLAG_USE_INVERTED_SPRITES) {
        vehicleEntry--;
    }
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_VERTICAL_SLOPES) {
        sint32 ecx = ((imageDirection / 8) ^ 2) + 76;
        sint32 ebx = (((imageDirection / 8) + 76) * vehicleEntry->var_16) + vehicleEntry->var_28;
        vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
    } else {
        vehicle_sprite_8(vehicle, imageDirection, z, vehicleEntry);
    }
}

// 6D4BE0
static void vehicle_sprite_20(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    if (vehicle->update_flags & VEHICLE_UPDATE_FLAG_USE_INVERTED_SPRITES) {
        vehicleEntry--;
    }
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_VERTICAL_SLOPES) {
        sint32 ecx = ((imageDirection / 8) ^ 2) + 80;
        sint32 ebx = (((imageDirection / 8) + 84) * vehicleEntry->var_16) + vehicleEntry->var_28;
        vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
    } else {
        vehicle_sprite_8(vehicle, imageDirection, z, vehicleEntry);
    }
}

// 6D4C40
static void vehicle_sprite_21(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    if (vehicle->update_flags & VEHICLE_UPDATE_FLAG_USE_INVERTED_SPRITES) {
        vehicleEntry--;
    }
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_VERTICAL_SLOPES) {
        sint32 ecx = ((imageDirection / 8) ^ 2) + 84;
        sint32 ebx = (((imageDirection / 8) + 92) * vehicleEntry->var_16) + vehicleEntry->var_28;
        vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
    } else {
        vehicle_sprite_8(vehicle, imageDirection, z, vehicleEntry);
    }
}

// 6D4CA0
static void vehicle_sprite_22(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    if (vehicle->update_flags & VEHICLE_UPDATE_FLAG_USE_INVERTED_SPRITES) {
        vehicleEntry--;
    }
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_VERTICAL_SLOPES) {
        sint32 ecx = ((imageDirection / 8) ^ 2) + 88;
        sint32 ebx = (((imageDirection / 8) + 100) * vehicleEntry->var_16) + vehicleEntry->var_28;
        vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
    } else {
        vehicle_sprite_8(vehicle, imageDirection, z, vehicleEntry);
    }
}

// 6D4D00
static void vehicle_sprite_23(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    if (vehicle->update_flags & VEHICLE_UPDATE_FLAG_USE_INVERTED_SPRITES) {
        vehicleEntry--;
    }
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_VERTICAL_SLOPES) {
        sint32 ecx = ((imageDirection / 8) ^ 2) + 92;
        sint32 ebx = (((imageDirection / 8) + 108) * vehicleEntry->var_16) + vehicleEntry->var_28;
        vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
    } else {
        vehicle_sprite_8(vehicle, imageDirection, z, vehicleEntry);
    }
}

// 6D51A5
static void vehicle_sprite_24(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    if (vehicle->update_flags & VEHICLE_UPDATE_FLAG_USE_INVERTED_SPRITES) {
        vehicleEntry--;
    }
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_CORKSCREWS) {
        sint32 eax = ((vehicle->vehicle_sprite_type - 24) * 4);
        sint32 ecx = (imageDirection / 8) + eax + 144;
        sint32 ebx = (((imageDirection / 8) + eax) * vehicleEntry->var_16) + vehicleEntry->var_4C;
        vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
    } else {
        vehicle_sprite_paint_6D51DE(vehicle, imageDirection, z, vehicleEntry);
    }
}

// 6D4D67
static void vehicle_sprite_50_0(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_DIAGONAL_SLOPES) {
        sint32 ecx = imageDirection / 2;
        sint32 ebx = ((imageDirection / 8) * vehicleEntry->var_16) + vehicleEntry->var_2C;
        vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
    } else {
        vehicle_sprite_0(vehicle, imageDirection, z, vehicleEntry);
    }
}

// 6D4DB5
static void vehicle_sprite_50_1(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_DIAGONAL_GENTLE_SLOPE_BANKED_TRANSITIONS) {
        sint32 ecx = imageDirection / 2;
        sint32 ebx = ((imageDirection / 8) * vehicleEntry->var_16) + vehicleEntry->var_3C;
        vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
    } else {
        vehicle_sprite_50_0(vehicle, imageDirection, z, vehicleEntry);
    }
}

// 6D4DD3
static void vehicle_sprite_50_3(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_DIAGONAL_GENTLE_SLOPE_BANKED_TRANSITIONS) {
        sint32 ecx = imageDirection / 2;
        sint32 ebx = (((imageDirection / 8) + 4) * vehicleEntry->var_16) + vehicleEntry->var_3C;
        vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
    } else {
        vehicle_sprite_50_0(vehicle, imageDirection, z, vehicleEntry);
    }
}

// 6D4D60
static void vehicle_sprite_50(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    // 0x009A3D44:
    switch (vehicle->bank_rotation) {
        case 0:  vehicle_sprite_50_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 1:  vehicle_sprite_50_1(vehicle, imageDirection, z, vehicleEntry); break;
        case 2:  vehicle_sprite_50_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 3:  vehicle_sprite_50_3(vehicle, imageDirection, z, vehicleEntry); break;
        case 4:  vehicle_sprite_50_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 5:  vehicle_sprite_50_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 6:  vehicle_sprite_50_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 7:  vehicle_sprite_50_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 8:  vehicle_sprite_50_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 9:  vehicle_sprite_50_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 10: vehicle_sprite_50_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 11: vehicle_sprite_50_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 12: vehicle_sprite_50_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 13: vehicle_sprite_50_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 14: vehicle_sprite_50_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 15: vehicle_sprite_50_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 16: vehicle_sprite_50_1(vehicle, imageDirection, z, vehicleEntry); break;
        case 17: vehicle_sprite_50_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 18: vehicle_sprite_50_3(vehicle, imageDirection, z, vehicleEntry); break;
        case 19: vehicle_sprite_50_0(vehicle, imageDirection, z, vehicleEntry); break;
    }
}

// 6D4E3A
static void vehicle_sprite_51(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_DIAGONAL_SLOPES) {
        sint32 ecx = (imageDirection / 8) + 100;
        sint32 ebx = (((imageDirection / 8) + 8) * vehicleEntry->var_16) + vehicleEntry->var_2C;
        vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
    } else {
        vehicle_sprite_0(vehicle, imageDirection, z, vehicleEntry);
    }
}

// 6D4E8F
static void vehicle_sprite_52(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_DIAGONAL_SLOPES) {
        sint32 ecx = (imageDirection / 8) + 104;
        sint32 ebx = (((imageDirection / 8) + 16) * vehicleEntry->var_16) + vehicleEntry->var_2C;
        vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
    } else {
        vehicle_sprite_0(vehicle, imageDirection, z, vehicleEntry);
    }
}

// 6D4D90
static void vehicle_sprite_53_0(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_DIAGONAL_SLOPES) {
        sint32 ecx = imageDirection / 2;
        sint32 ebx = (((imageDirection / 8) + 4) * vehicleEntry->var_16) + vehicleEntry->var_2C;
        vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
    } else {
        vehicle_sprite_0(vehicle, imageDirection, z, vehicleEntry);
    }
}

// 6D4DF4
static void vehicle_sprite_53_1(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_DIAGONAL_GENTLE_SLOPE_BANKED_TRANSITIONS) {
        sint32 ecx = imageDirection / 2;
        sint32 ebx = (((imageDirection / 8) + 8) * vehicleEntry->var_16) + vehicleEntry->var_3C;
        vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
    } else {
        vehicle_sprite_53_0(vehicle, imageDirection, z, vehicleEntry);
    }
}

// 6D4E15
static void vehicle_sprite_53_3(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_DIAGONAL_GENTLE_SLOPE_BANKED_TRANSITIONS) {
        sint32 ecx = imageDirection / 2;
        sint32 ebx = (((imageDirection / 8) + 12) * vehicleEntry->var_16) + vehicleEntry->var_3C;
        vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
    } else {
        vehicle_sprite_53_0(vehicle, imageDirection, z, vehicleEntry);
    }
}

// 6D4D89
static void vehicle_sprite_53(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    // 0x009A3D94:
    switch (vehicle->bank_rotation) {
        case 0:  vehicle_sprite_53_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 1:  vehicle_sprite_53_1(vehicle, imageDirection, z, vehicleEntry); break;
        case 2:  vehicle_sprite_53_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 3:  vehicle_sprite_53_3(vehicle, imageDirection, z, vehicleEntry); break;
        case 4:  vehicle_sprite_53_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 5:  vehicle_sprite_53_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 6:  vehicle_sprite_53_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 7:  vehicle_sprite_53_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 8:  vehicle_sprite_53_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 9:  vehicle_sprite_53_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 10: vehicle_sprite_53_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 11: vehicle_sprite_53_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 12: vehicle_sprite_53_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 13: vehicle_sprite_53_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 14: vehicle_sprite_53_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 15: vehicle_sprite_53_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 16: vehicle_sprite_53_1(vehicle, imageDirection, z, vehicleEntry); break;
        case 17: vehicle_sprite_53_0(vehicle, imageDirection, z, vehicleEntry); break;
        case 18: vehicle_sprite_53_3(vehicle, imageDirection, z, vehicleEntry); break;
        case 19: vehicle_sprite_53_0(vehicle, imageDirection, z, vehicleEntry); break;
    }
}

// 6D4E63
static void vehicle_sprite_54(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_DIAGONAL_SLOPES) {
        sint32 ecx = ((imageDirection / 8) ^ 2) + 100;
        sint32 ebx = (((imageDirection / 8) + 12) * vehicleEntry->var_16) + vehicleEntry->var_2C;
        vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
    } else {
        vehicle_sprite_0(vehicle, imageDirection, z, vehicleEntry);
    }
}

// 6D4EB8
static void vehicle_sprite_55(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_DIAGONAL_SLOPES) {
        sint32 ecx = ((imageDirection / 8) ^ 2) + 104;
        sint32 ebx = (((imageDirection / 8) + 20) * vehicleEntry->var_16) + vehicleEntry->var_2C;
        vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
    } else {
        vehicle_sprite_0(vehicle, imageDirection, z, vehicleEntry);
    }
}

// 6D47DA
static void vehicle_sprite_56(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    vehicleEntry--;
    vehicle_sprite_6(vehicle, imageDirection, z, vehicleEntry);
}

// 6D4A02
static void vehicle_sprite_57(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    vehicleEntry--;
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_STEEP_SLOPES) {
        sint32 ecx = ((imageDirection / 4) ^ 4) + 32;
        sint32 ebx = (((imageDirection / 4) + 8) * vehicleEntry->var_16) + vehicleEntry->var_24;
        vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
    } else {
        vehicle_sprite_6(vehicle, imageDirection, z, vehicleEntry);
    }
}

// 6D4A56
static void vehicle_sprite_58(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    vehicleEntry--;
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_STEEP_SLOPES) {
        sint32 ecx = ((imageDirection / 2) ^ 8) + 40;
        sint32 ebx = ((imageDirection + 48) * vehicleEntry->var_16) + vehicleEntry->var_24;
        vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
    } else {
        vehicle_sprite_6(vehicle, imageDirection, z, vehicleEntry);
    }
}

// 6D4773
static void vehicle_sprite_59(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry)
{
    if (vehicleEntry->sprite_flags & VEHICLE_SPRITE_FLAG_14) {
        sint32 ecx = (imageDirection / 2) + 16;
        sint32 ebx = (imageDirection * vehicleEntry->var_16) + vehicleEntry->var_4C;
        vehicle_sprite_paint_6D520E(vehicle, ebx, ecx, z, vehicleEntry);
    } else {
        vehicle_sprite_2(vehicle, imageDirection, z, vehicleEntry);
    }
}

// 0x009A3B14:
typedef void (*vehicle_sprite_func)(rct_vehicle *vehicle, sint32 imageDirection, sint32 z, const rct_ride_entry_vehicle *vehicleEntry);
vehicle_sprite_func vehicle_sprite_funcs[] = {
    vehicle_sprite_0,
    vehicle_sprite_1,
    vehicle_sprite_2,
    vehicle_sprite_3,
    vehicle_sprite_4,
    vehicle_sprite_5,
    vehicle_sprite_6,
    vehicle_sprite_7,
    vehicle_sprite_8,
    vehicle_sprite_9,
    vehicle_sprite_10,
    vehicle_sprite_11,
    vehicle_sprite_12,
    vehicle_sprite_13,
    vehicle_sprite_14,
    vehicle_sprite_15,
    vehicle_sprite_16,
    vehicle_sprite_17,
    vehicle_sprite_18,
    vehicle_sprite_19,
    vehicle_sprite_20,
    vehicle_sprite_21,
    vehicle_sprite_22,
    vehicle_sprite_23,
    vehicle_sprite_24,
    vehicle_sprite_24,
    vehicle_sprite_24,
    vehicle_sprite_24,
    vehicle_sprite_24,
    vehicle_sprite_24,
    vehicle_sprite_24,
    vehicle_sprite_24,
    vehicle_sprite_24,
    vehicle_sprite_24,
    vehicle_sprite_24,
    vehicle_sprite_24,
    vehicle_sprite_24,
    vehicle_sprite_24,
    vehicle_sprite_24,
    vehicle_sprite_24,
    vehicle_sprite_24,
    vehicle_sprite_24,
    vehicle_sprite_24,
    vehicle_sprite_24,
    vehicle_sprite_0,
    vehicle_sprite_0,
    vehicle_sprite_0,
    vehicle_sprite_0,
    vehicle_sprite_0,
    vehicle_sprite_0,
    vehicle_sprite_50,
    vehicle_sprite_51,
    vehicle_sprite_52,
    vehicle_sprite_53,
    vehicle_sprite_54,
    vehicle_sprite_55,
    vehicle_sprite_56,
    vehicle_sprite_57,
    vehicle_sprite_58,
    vehicle_sprite_59,
};

/**
 *
 *  rct2: 0x006D5600
 */
static void vehicle_visual_splash1_effect(sint32 z, rct_vehicle *vehicle, const rct_ride_entry_vehicle *vehicleEntry)
{
    if ((vehicle->track_type >> 2) != TRACK_ELEM_WATER_SPLASH) {
        return;
    }
    if (vehicle->track_progress < 48) {
        return;
    }
    if (vehicle->track_progress >= 112) {
        return;
    }
    if (vehicle_get_head(vehicle)->velocity <= 0x50000) {
        return;
    }
    sint32 image_id = 29014 + ((((vehicle->sprite_direction / 8) + get_current_rotation()) & 3) * 8) + ((gCurrentTicks / 2) & 7);
    sub_98199C(image_id, 0, 0, 0, 0, 0, z, 0, 0, z, get_current_rotation());
}

/**
 *
 *  rct2: 0x006D5696
 */
static void vehicle_visual_splash2_effect(sint32 z, rct_vehicle *vehicle, const rct_ride_entry_vehicle *vehicleEntry)
{
    if (vehicle->sprite_direction & 7) {
        return;
    }
    if (vehicle->vehicle_sprite_type != 0) {
        return;
    }
    if (vehicle->velocity <= 0x50000) {
        return;
    }
    sint32 image_id = 29046 + ((((vehicle->sprite_direction / 8) + get_current_rotation()) & 3) * 8) + ((gCurrentTicks / 2) & 7);
    sub_98199C(image_id, 0, 0, 0, 0, 0, z, 0, 0, z, get_current_rotation());
}

/**
 *
 *  rct2: 0x006D57EE
 */
static void vehicle_visual_splash3_effect(sint32 z, rct_vehicle *vehicle, const rct_ride_entry_vehicle *vehicleEntry)
{
    if (vehicle->sprite_direction & 7) {
        return;
    }
    if (vehicle->vehicle_sprite_type != 0) {
        return;
    }
    if (vehicle->velocity <= 0x50000) {
        return;
    }
    sint32 image_id = 29014 + ((((vehicle->sprite_direction / 8) + get_current_rotation()) & 3) * 8) + ((gCurrentTicks / 2) & 7);
    sub_98199C(image_id, 0, 0, 0, 0, 0, z, 0, 0, z, get_current_rotation());
}

/**
 *
 *  rct2: 0x006D5783
 */
static void vehicle_visual_splash4_effect(sint32 z, rct_vehicle *vehicle, const rct_ride_entry_vehicle *vehicleEntry)
{
    rct_vehicle *vehicle2 = GET_VEHICLE(vehicle->prev_vehicle_on_ride);
    if (vehicle2->velocity <= 0x50000) {
        return;
    }
    if (vehicle->sprite_direction & 7) {
        return;
    }
    if (vehicle->vehicle_sprite_type != 0) {
        return;
    }
    sint32 image_id = 29078 + ((((vehicle->sprite_direction / 8) + get_current_rotation()) & 3) * 8) + ((gCurrentTicks / 2) & 7);
    sub_98199C(image_id, 0, 0, 1, 1, 0, z, 0, 0, z, get_current_rotation());
}

/**
 *
 *  rct2: 0x006D5701
 */
static void vehicle_visual_splash5_effect(sint32 z, rct_vehicle *vehicle, const rct_ride_entry_vehicle *vehicleEntry)
{
    rct_vehicle *vehicle2 = GET_VEHICLE(vehicle->prev_vehicle_on_ride);
    if (vehicle2->velocity <= 0x50000) {
        return;
    }
    if (vehicle->sprite_direction & 7) {
        return;
    }
    if (vehicle->vehicle_sprite_type != 0) {
        return;
    }
    if (!track_element_is_covered(vehicle->track_type >> 2)) {
        return;
    }
    sint32 image_id = 29078 + ((((vehicle->sprite_direction / 8) + get_current_rotation()) & 3) * 8) + ((gCurrentTicks / 2) & 7);
    sub_98199C(image_id, 0, 0, 1, 1, 0, z, 0, 0, z, get_current_rotation());
}

void vehicle_visual_splash_effect(sint32 z, rct_vehicle *vehicle, const rct_ride_entry_vehicle *vehicleEntry)
{
    switch (vehicleEntry->effect_visual) {
        case 1: /* nullsub */ break;
        case VEHICLE_VISUAL_SPLASH1_EFFECT: vehicle_visual_splash1_effect(z, vehicle, vehicleEntry); break;
        case VEHICLE_VISUAL_SPLASH2_EFFECT: vehicle_visual_splash2_effect(z, vehicle, vehicleEntry); break;
        case VEHICLE_VISUAL_SPLASH3_EFFECT: vehicle_visual_splash3_effect(z, vehicle, vehicleEntry); break;
        case VEHICLE_VISUAL_SPLASH4_EFFECT: vehicle_visual_splash4_effect(z, vehicle, vehicleEntry); break;
        case VEHICLE_VISUAL_SPLASH5_EFFECT: vehicle_visual_splash5_effect(z, vehicle, vehicleEntry); break;
        default: assert(false); break;
    }
}

/**
 *
 *  rct2: 0x006D45F8
 */
void vehicle_visual_default(sint32 x, sint32 imageDirection, sint32 y, sint32 z, rct_vehicle *vehicle, const rct_ride_entry_vehicle *vehicleEntry)
{
    assert(vehicle->vehicle_sprite_type < countof(vehicle_sprite_funcs));
    if (vehicle->vehicle_sprite_type < countof(vehicle_sprite_funcs)) {
        vehicle_sprite_funcs[vehicle->vehicle_sprite_type](vehicle, imageDirection, z, vehicleEntry);
    }
}

/**
 *
 *  rct2: 0x006D4244
 */
void vehicle_paint(rct_vehicle *vehicle, sint32 imageDirection)
{
    rct_ride_entry *rideEntry = 0;
    const rct_ride_entry_vehicle *vehicleEntry;

    sint32 x = vehicle->x;
    sint32 y = vehicle->y;
    sint32 z = vehicle->z;

    if (vehicle->flags & SPRITE_FLAGS_IS_CRASHED_VEHICLE_SPRITE) {
        uint32 ebx = 22965 + vehicle->var_C5;
        sub_98197C(ebx, 0, 0, 1, 1, 0, z, 0, 0, z + 2, get_current_rotation());
        return;
    }

    if (vehicle->ride_subtype == RIDE_ENTRY_INDEX_NULL) {
        vehicleEntry = &CableLiftVehicle;
    } else {
        rideEntry = get_ride_entry(vehicle->ride_subtype);
        vehicleEntry = &rideEntry->vehicles[vehicle->vehicle_type];

        if (vehicle->update_flags & VEHICLE_UPDATE_FLAG_USE_INVERTED_SPRITES) {
            vehicleEntry++;
            z += 16;
        }
    }

    switch (vehicleEntry->car_visual) {
    case VEHICLE_VISUAL_DEFAULT:                        vehicle_visual_default(x, imageDirection, y, z, vehicle, vehicleEntry); break;
    case VEHICLE_VISUAL_LAUNCHED_FREEFALL:              vehicle_visual_launched_freefall(x, imageDirection, y, z, vehicle, vehicleEntry); break;
    case VEHICLE_VISUAL_OBSERVATION_TOWER:              vehicle_visual_observation_tower(x, imageDirection, y, z, vehicle, vehicleEntry); break;
    case VEHICLE_VISUAL_RIVER_RAPIDS:                   vehicle_visual_river_rapids(x, imageDirection, y, z, vehicle, vehicleEntry); break;
    case VEHICLE_VISUAL_MINI_GOLF_PLAYER:               vehicle_visual_mini_golf_player(x, imageDirection, y, z, vehicle); break;
    case VEHICLE_VISUAL_MINI_GOLF_BALL:                 vehicle_visual_mini_golf_ball(x, imageDirection, y, z, vehicle); break;
    case VEHICLE_VISUAL_REVERSER:                       vehicle_visual_reverser(x, imageDirection, y, z, vehicle, vehicleEntry); break;
    case VEHICLE_VISUAL_SPLASH_BOATS_OR_WATER_COASTER:  vehicle_visual_splash_boats_or_water_coaster(x, imageDirection, y, z, vehicle, vehicleEntry); break;
    case VEHICLE_VISUAL_ROTO_DROP:                      vehicle_visual_roto_drop(x, imageDirection, y, z, vehicle, vehicleEntry); break;
    case VEHICLE_VISUAL_VIRGINIA_REEL:                  vehicle_visual_virginia_reel(x, imageDirection, y, z, vehicle, vehicleEntry); break;
    case VEHICLE_VISUAL_SUBMARINE:                      vehicle_visual_submarine(x, imageDirection, y, z, vehicle, vehicleEntry); break;
    }
}
